@@ -194,7 +194,7 @@ void print(WWWconnection& con, dbCursor<T>& cursor) {
 template<class T>
 void print(WWWconnection& con, dbArray<dbReference<T> > const& arr) {
     dbCursor<T> cursor;
-    for (int i = 0, n = arr.length(); i < n; i++) {
+    for (int i = 0, n = (int)arr.length(); i < n; i++) {
         cursor.at(arr[i])->print(con);
     }
 }
