@@ -76,8 +76,8 @@ int __cdecl main()
     if (s == NULL) {
         message("Failed to connect to application");
     }
-    *(int*)buf = buf_size;
-    buf[4] = src - buf - 3;
+    *(int*)buf = (int)buf_size;
+    buf[4] = (char)(src - buf - 3);
     if (!s->write(buf, buf_size)) {
         message("Failed to send query to application");
     }
