@@ -40,6 +40,7 @@ struct column_binding {
     int     arr_len;
     cli_column_get_ex get_fnc;
     cli_column_set_ex set_fnc;
+    void* user_data;
 
     ~column_binding() {
         delete[] name;
@@ -57,7 +58,7 @@ struct statement_desc {
     column_binding*    columns;
     parameter_binding* params;
     session_desc*      session;
-    bool               for_update;
+    int                cursor_type;
     bool               prepared;
     bool               updated;
     bool               autoincrement;
@@ -76,7 +77,7 @@ struct statement_desc {
             next_cb = cb->next;
             delete cb;
         }
-        if (buf != NULL && buf_size > DEFAULT_BUF_SIZE) { 
+        if (buf != NULL) { 
             delete[] buf;
             buf_size = 0;
             buf = NULL;
@@ -127,16 +128,22 @@ class descriptor_table {
 
   public:
     descriptor_table() {
-        int i;
         descriptor_table_size = 16;
         table = new T*[descriptor_table_size];
         T* next = NULL;
-        for (i = 0; i < descriptor_table_size; i++) {
+        for (int i = 0; i < descriptor_table_size; i++) {
             table[i] = next = new T(i, next);
         }
         free_desc = next;
     }
 
+    ~descriptor_table() {
+        for (int i = 0; i < descriptor_table_size; i++) {
+            delete table[i];
+        }
+        delete[] table;
+    }
+
     T* get(int desc) {
         dbCriticalSection cs(mutex);
         return (desc >= descriptor_table_size) ? (T*)0 : table[desc];
@@ -172,11 +179,42 @@ class descriptor_table {
 static descriptor_table<session_desc>   sessions;
 static descriptor_table<statement_desc> statements;
 
+static int cli_close_internal(session_desc* s)
+{
+    statement_desc *stmt, *next;
+    cli_request req;
+    req.length  = sizeof(req);
+    req.cmd     = cli_cmd_close_session;
+    req.stmt_id = 0;
+    req.pack();
+    int result = cli_ok;
+    if (!s->sock->write(&req, sizeof req)) {
+        result = cli_network_error;
+    }
+    delete s->sock;
+    s->sock = NULL;     
+    for (stmt = s->stmts; stmt != NULL; stmt = next) {
+        next = stmt->next;
+        stmt->free();
+        statements.free(stmt);
+    }
+    sessions.free(s);
+    return result;
+}
+
 class connection_pool { 
     session_desc* connection_chain;
     dbMutex       mutex;
     
   public:
+    connection_pool() { 
+        connection_chain = NULL;
+    }
+
+    ~connection_pool() { 
+        close();
+    }
+
     session_desc* new_connection(char   const* server_url,
                       char_t const* user_name,
                       char_t const* password)
@@ -187,7 +225,7 @@ class connection_pool {
                 STRCMP(desc->user, user_name) == 0 &&
                 STRCMP(desc->password, password) == 0)
             {
-                connection_chain = desc->next;
+                connection_chain = desc->next_pooled;
                 return desc;
             }
         }
@@ -197,17 +235,19 @@ class connection_pool {
     void release(session_desc* desc) 
     {
         dbCriticalSection cs(mutex);
-        desc->next = connection_chain;
+        desc->next_pooled = connection_chain;
         connection_chain = desc;
     }
 
     void close() { 
         dbCriticalSection cs(mutex);
-        for (session_desc* desc = connection_chain; desc != NULL; desc = desc->next_pooled) {
+        session_desc *desc, *next;
+        for (desc = connection_chain; desc != NULL; desc = next) {
             desc->pool = NULL;
             delete[] desc->user;
             delete[] desc->password;
-            cli_close(desc->id);
+            next = desc->next_pooled;
+            cli_close_internal(desc);
         }
         connection_chain = NULL;
     }
@@ -252,7 +292,7 @@ int cli_open(char const*   server_url,
             if (end == NULL) { 
                 end = start + strlen(start);
             }
-            int len = end - start;
+            int len = (int)(end - start);
             char* addr = new char[len+1];
             memcpy(addr, start, len);
             addr[len] = '\0';
@@ -271,7 +311,7 @@ int cli_open(char const*   server_url,
 
     if (!sock->is_ok()) {
         char_t buf[256];
-        sock->get_error_text(buf, sizeof buf);
+        sock->get_error_text(buf, itemsof(buf));
         fprintf(stderr, "Failed to connect to server: %s\n", buf);
         delete sock;
         return cli_connection_refused;
@@ -281,7 +321,7 @@ int cli_open(char const*   server_url,
     dbSmallBuffer<char> buf(msg_size);
     char* p = buf;
     cli_request* req = (cli_request*)p;
-    req->length  = msg_size;
+    req->length  = (int4)msg_size;
     req->cmd     = cli_cmd_login;
     req->stmt_id = 0;
     p += sizeof(cli_request);
@@ -316,7 +356,6 @@ int cli_open(char const*   server_url,
 
 int cli_close(int session)
 {
-    statement_desc *stmt, *next;
     session_desc* s = sessions.get(session);
     if (s == NULL) {
         return cli_bad_descriptor;
@@ -325,25 +364,7 @@ int cli_close(int session)
         s->pool->release(s);
         return cli_ok;
     }
-
-    cli_request req;
-    req.length  = sizeof(req);
-    req.cmd     = cli_cmd_close_session;
-    req.stmt_id = 0;
-    req.pack();
-    int result = cli_ok;
-    if (!s->sock->write(&req, sizeof req)) {
-        result = cli_network_error;
-    }
-    delete s->sock;
-    s->sock = NULL;     
-    for (stmt = s->stmts; stmt != NULL; stmt = next) {
-        next = stmt->next;
-        stmt->free();
-        statements.free(stmt);
-    }
-    sessions.free(s);
-    return result;
+    return cli_close_internal(s);
 }
 
 void cli_clear_connection_pool()
@@ -363,7 +384,7 @@ int cli_statement(int session, char_t const* stmt_str)
     stmt->columns = NULL;
     stmt->params = NULL;
     stmt->session = s;
-    stmt->for_update = 0;
+    stmt->cursor_type = cli_cursor_view_only;
     stmt->prepared = false;
     stmt->n_params = 0;
     stmt->n_columns = 0;
@@ -400,7 +421,7 @@ int cli_statement(int session, char_t const* stmt_str)
                 return cli_bad_statement;
             }
             parameter_binding* pb = new parameter_binding;
-            int len = p - q;
+            int len = (int)(p - q);
             pb->name = new char_t[len+1];
             memcpy(pb->name, q, len*sizeof(char_t));
             pb->name[len] = '\0';
@@ -415,7 +436,7 @@ int cli_statement(int session, char_t const* stmt_str)
     if (dst == stmt->stmt || *(dst-1) != '\0') {
         *dst++ = '\0';
     }
-    stmt->stmt_len = dst - stmt->stmt;
+    stmt->stmt_len = (int)(dst - stmt->stmt);
     *last = NULL;
     return stmt->id;
 }
@@ -462,7 +483,7 @@ int cli_column(int           statement,
     }
     s->prepared = false;
     column_binding* cb = new column_binding;
-    int len = STRLEN(column_name) + 1;
+    int len = (int)STRLEN(column_name) + 1;
     cb->name = new char_t[len];
     s->columns_len += len;
     cb->next = s->columns;
@@ -485,7 +506,7 @@ int cli_array_column(int            statement,
                      cli_column_get get)
 {
     return cli_array_column_ex(statement, column_name, var_type, var_ptr, 
-                               (cli_column_set_ex)set, (cli_column_get_ex)get);
+                               (cli_column_set_ex)set, (cli_column_get_ex)get, NULL);
 }
 
 int cli_array_column_ex(int               statement,
@@ -493,7 +514,8 @@ int cli_array_column_ex(int               statement,
                         int               var_type,
                         void*             var_ptr,
                         cli_column_set_ex set,
-                        cli_column_get_ex get)
+                        cli_column_get_ex get, 
+                        void*             user_data)
 {
     statement_desc* s = statements.get(statement);
     if (s == NULL) {
@@ -504,7 +526,7 @@ int cli_array_column_ex(int               statement,
     }
     s->prepared = false;
     column_binding* cb = new column_binding;
-    int len = STRLEN(column_name) + 1;
+    int len = (int)STRLEN(column_name) + 1;
     cb->name = new char_t[len];
     s->columns_len += len;
     cb->next = s->columns;
@@ -516,10 +538,21 @@ int cli_array_column_ex(int               statement,
     cb->var_ptr = var_ptr;
     cb->set_fnc = set;
     cb->get_fnc = get;
+    cb->user_data = user_data;
     return cli_ok;
 }
 
-int cli_fetch(int statement, int for_update)
+int cli_fetch_ex(int statement, int cursor_type, cli_cardinality_t* n_fetched_records)
+{
+    int rc = cli_fetch(statement,  cursor_type);
+    if (rc >= 0) { 
+        *n_fetched_records = rc;
+        return cli_ok;
+    }
+    return rc;
+}
+
+int cli_fetch(int statement, int cursor_type)
 {
     parameter_binding* pb;
     column_binding*    cb;
@@ -529,7 +562,7 @@ int cli_fetch(int statement, int for_update)
     if (stmt == NULL) {
         return cli_bad_descriptor;
     }
-    stmt->for_update = for_update;
+    stmt->cursor_type = cursor_type;
     int msg_size = sizeof(cli_request) + 1;
     if (!stmt->prepared) {
         msg_size += 4 + stmt->stmt_len*sizeof(char_t) + stmt->n_params;
@@ -540,11 +573,11 @@ int cli_fetch(int statement, int for_update)
             return cli_unbound_parameter;
         }
         if (pb->var_type == cli_asciiz) {
-            msg_size += (-msg_size & (sizeof(char_t)-1)) + (STRLEN((char_t*)pb->var_ptr) + 1)*sizeof(char_t);
+            msg_size += (int)((-msg_size & (sizeof(char_t)-1)) + (STRLEN((char_t*)pb->var_ptr) + 1)*sizeof(char_t));
         } else if (pb->var_type == cli_pasciiz) {
-            msg_size += (-msg_size & (sizeof(char_t)-1)) + (STRLEN(*(char_t**)pb->var_ptr) + 1)*sizeof(char_t);
+            msg_size += (int)((-msg_size & (sizeof(char_t)-1)) + (STRLEN(*(char_t**)pb->var_ptr) + 1)*sizeof(char_t));
         } else if (pb->var_type == cli_cstring) {
-            msg_size += ((cli_cstring_t*)pb->var_ptr)->len*sizeof(char_t) + 4;
+            msg_size += (int)(((cli_cstring_t*)pb->var_ptr)->len*sizeof(char_t) + 4);
         } else {
             msg_size += sizeof_type[pb->var_type];
         }
@@ -562,7 +595,7 @@ int cli_fetch(int statement, int for_update)
     if (!stmt->prepared) {
         *p++ = stmt->n_params;
         *p++ = stmt->n_columns;
-        p = pack2(p, stmt->stmt_len*sizeof(char_t) + stmt->n_params);
+        p = pack2(p, (int2)(stmt->stmt_len*sizeof(char_t) + stmt->n_params));
         pb = stmt->params;
         char* end = p + stmt->stmt_len*sizeof(char_t) + stmt->n_params;
         char_t* src = stmt->stmt;
@@ -580,18 +613,18 @@ int cli_fetch(int statement, int for_update)
             p = pack_str(p, cb->name);
         }
     }
-    *p++ = for_update;
+    *p++ = cursor_type;
     for (pb = stmt->params; pb != NULL; pb = pb->next) {
         switch (pb->var_type) {
           case cli_asciiz:
-            p += -(int)p & (sizeof(char_t)-1);
+            p += -(int)(size_t)p & (sizeof(char_t)-1);
             p = pack_str(p, (char_t*)pb->var_ptr);
             continue;
           case cli_rectangle:
             p = pack_rectangle(p, (cli_rectangle_t*)pb->var_ptr);
             continue;
           case cli_pasciiz:
-            p += -(int)p & (sizeof(char_t)-1);
+            p += -(int)(size_t)p & (sizeof(char_t)-1);
             p = pack_str(p, *(char_t**)pb->var_ptr);
             continue;
           case cli_cstring:
@@ -642,7 +675,7 @@ static int cli_send_columns(int statement, int cmd)
     if (s == NULL) {
         return cli_bad_descriptor;
     }
-    long msg_size = sizeof(cli_request);
+    size_t msg_size = sizeof(cli_request);
     if (cmd == cli_cmd_update) {
         if (!s->prepared) {
             return cli_not_fetched;
@@ -653,7 +686,7 @@ static int cli_send_columns(int statement, int cmd)
         if (s->updated) {
             return cli_already_updated;
         } 
-        if (!s->for_update) {
+        if (s->cursor_type != cli_cursor_for_update) {
             return cli_not_update_mode;
         }
     } else {
@@ -666,7 +699,7 @@ static int cli_send_columns(int statement, int cmd)
     for (cb = s->columns; cb != NULL; cb = cb->next) {
         if (cb->get_fnc != NULL) {
             cb->arr_ptr = cb->get_fnc(cb->var_type, cb->var_ptr, &cb->arr_len, 
-                                      cb->name, statement);
+                                      cb->name, statement, cb->user_data);
             len = cb->arr_len;
             msg_size += 4;
             if (cb->var_type == cli_array_of_string) {
@@ -714,7 +747,7 @@ static int cli_send_columns(int statement, int cmd)
     dbSmallBuffer<char> buf(msg_size);
     char* p = buf;
     cli_request* req = (cli_request*)p;
-    req->length  = msg_size;
+    req->length  = (int4)msg_size;
     req->cmd     = cmd;
     req->stmt_id = statement;
     p += sizeof(cli_request);
@@ -771,7 +804,7 @@ static int cli_send_columns(int statement, int cmd)
                 }
             }
         } else if (cb->var_type == cli_asciiz) {
-            p = pack4(p, STRLEN((char_t*)src)+1);
+            p = pack4(p, (int4)STRLEN((char_t*)src)+1);
             p = pack_str(p, (char_t*)src);
         } else if (cb->var_type == cli_cstring) {
             int len = ((cli_cstring_t*)src)->len;
@@ -779,7 +812,7 @@ static int cli_send_columns(int statement, int cmd)
             p = pack_str(p, ((cli_cstring_t*)src)->ptr, len);
         } else if (cb->var_type == cli_pasciiz) {
             src = *(char**)src;
-            p = pack4(p, STRLEN((char_t*)src)+1);
+            p = pack4(p, (int4)STRLEN((char_t*)src)+1);
             p = pack_str(p, (char_t*)src);
         } else if (cb->var_type == cli_rectangle) {
             p = pack_rectangle(p, (cli_rectangle_t*)src);
@@ -799,7 +832,7 @@ static int cli_send_columns(int statement, int cmd)
             }
         }
     }
-    assert(p - buf.base() == msg_size);
+    assert((size_t)(p - buf.base()) == msg_size);
     req->pack();
     if (!s->session->sock->write(buf, msg_size)) {
         return cli_network_error;
@@ -807,6 +840,11 @@ static int cli_send_columns(int statement, int cmd)
     return cli_ok;
 }
 
+int cli_batch_insert(int statement, cli_oid_t* oid)
+{
+    return cli_insert(statement, oid);
+}
+
 int cli_insert(int statement, cli_oid_t* oid)
 {
     int rc = cli_send_columns(statement, cli_cmd_insert);
@@ -901,7 +939,6 @@ static int cli_get(int statement, int cmd, cli_oid_t value = 0)
     int result = cli_ok;
     if (cmd == cli_cmd_seek) { 
         s->oid = value;
-        result = unpack_oid(p);
     } else { 
         s->oid = unpack_oid(p);
         if (s->oid == 0) { 
@@ -919,14 +956,14 @@ static int cli_get(int statement, int cmd, cli_oid_t value = 0)
         if (cb->set_fnc != NULL) {
             int len = unpack4(p);
             p += 4;
-            char* dst = (char*)cb->set_fnc(type, cb->var_ptr, len, cb->name, statement, p);
+            char* dst = (char*)cb->set_fnc(type, cb->var_ptr, len, cb->name, statement, p, cb->user_data);
             if (dst == NULL) {
                 continue;
             }
             if (type == cli_array_of_string) { 
                 char_t** s = (char_t**)dst;
 #ifdef UNICODE
-                char_t* q = (char_t*)((long)p & ~(sizeof(char_t)-1));
+                char_t* q = (char_t*)((size_t)p & ~(sizeof(char_t)-1));
                 while (--len >= 0) {
                     char* p1 = unpack_str(q, p);
                     *s++  = q;
@@ -939,7 +976,7 @@ static int cli_get(int statement, int cmd, cli_oid_t value = 0)
                     p += strlen(p) + 1;
                 }
 #endif
-            } else if (type >= cli_array_of_oid && type <= cli_array_of_string) {
+            } else if (type >= cli_array_of_oid && type < cli_array_of_string) {
                 switch (sizeof_type[type-cli_array_of_oid]) {
                   case 2:
                     while (--len >= 0) {
@@ -990,7 +1027,7 @@ static int cli_get(int statement, int cmd, cli_oid_t value = 0)
                         char_t** s = (char_t**)dst;
                         len -= n;
 #ifdef UNICODE
-                        char_t* q = (char_t*)((long)p & ~(sizeof(char_t)-1));
+                        char_t* q = (char_t*)((size_t)p & ~(sizeof(char_t)-1));
                         while (--n >= 0) {
                             char* p1 = unpack_str(q, p);
                             *s++  = q;
@@ -1048,7 +1085,7 @@ static int cli_get(int statement, int cmd, cli_oid_t value = 0)
                       case cli_cstring:
                         ((cli_cstring_t*)dst)->len = n;
 #ifdef UNICODE
-                        ((cli_cstring_t*)dst)->ptr = (char_t*)((long)p & ~(sizeof(char_t)-1));
+                        ((cli_cstring_t*)dst)->ptr = (char_t*)((size_t)p & ~(sizeof(char_t)-1));
                         unpack_str(((cli_cstring_t*)dst)->ptr, p, n);
 #else
                         ((cli_cstring_t*)dst)->ptr = p;
@@ -1177,6 +1214,11 @@ static int cli_send_command(int session, int statement, int cmd)
     return response;
 }
 
+int cli_close_cursor(int statement)
+{
+    return cli_ok;
+}
+
 int cli_free(int statement)
 {
     statement_desc* stmt = statements.get(statement);
@@ -1205,6 +1247,10 @@ int cli_free(int statement)
     return cli_ok;
 }
 
+int cli_exec_batch(int session)
+{
+    return cli_ok;
+}
 
 int cli_commit(int session)
 {
@@ -1227,15 +1273,31 @@ int cli_remove(int statement)
     if (s == NULL) {
         return cli_bad_descriptor;
     }
-    if (!s->for_update) {
+    if (s->cursor_type != cli_cursor_for_update) {
         return cli_not_update_mode;
     }
     return cli_send_command(s->session->id, s->id, cli_cmd_remove);
 }
 
+int cli_remove_current(int statement)
+{
+    statement_desc* s = statements.get(statement);
+    if (s == NULL) { 
+        return cli_bad_descriptor;
+    }
+    if (s->oid == 0) { 
+        return cli_not_found;
+    }
+    if (s->cursor_type != cli_cursor_for_update) {
+        return cli_not_update_mode;
+    }
+    return cli_send_command(s->session->id, s->id, cli_cmd_remove_current);
+}
+
+
 int cli_describe(int session, char_t const* table, cli_field_descriptor** fields)
 {
-    int len = sizeof(cli_request) + STRLEN(table)*sizeof(char_t) + 1;
+    int len = (int)(sizeof(cli_request) + STRLEN(table)*sizeof(char_t) + 1);
     dbSmallBuffer<char> buf(len);
     cli_request* req = (cli_request*)buf.base();
     req->length = len;
@@ -1291,6 +1353,11 @@ int cli_describe(int session, char_t const* table, cli_field_descriptor** fields
     return nFields;
 }
 
+int cli_describe_layout(int session, char_t const* table, cli_field_layout** fields, int* rec_size)
+{
+    return cli_not_implemented;
+}
+
 
 int cli_show_tables(int session, cli_table_descriptor** tables)
 {
@@ -1350,7 +1417,7 @@ static int cli_update_table(int cmd, int session, char_t const* tableName, int n
     if (s == NULL) {
         return cli_bad_descriptor;
     }
-    int size = sizeof(cli_request) + 4 + (STRLEN(tableName)+1)*sizeof(char_t);
+    size_t size = sizeof(cli_request) + 4 + (STRLEN(tableName)+1)*sizeof(char_t);
     for (i = 0; i < nColumns; i++) { 
         size += 2 + (STRLEN(columns[i].name)+3)*sizeof(char_t);
         if (columns[i].refTableName != NULL) { 
@@ -1362,7 +1429,7 @@ static int cli_update_table(int cmd, int session, char_t const* tableName, int n
     }
     dbSmallBuffer<char> buf(size);
     cli_request* req = (cli_request*)buf.base();
-    req->length  = size;
+    req->length  = (int4)size;
     req->cmd     = cmd;
     req->stmt_id = 0;
     char* dst = (char*)(req + 1);
@@ -1415,10 +1482,10 @@ int cli_drop_table(int session, char_t const* tableName)
     if (s == NULL) {
         return cli_bad_descriptor;
     }
-    int size = sizeof(cli_request) + (STRLEN(tableName)+1)*sizeof(char_t);
+    size_t size = sizeof(cli_request) + (STRLEN(tableName)+1)*sizeof(char_t);
     dbSmallBuffer<char> buf(size);
     cli_request* req = (cli_request*)buf.base();
-    req->length  = size;
+    req->length  = (int4)size;
     req->cmd     = cli_cmd_drop_table;
     req->stmt_id = 0;
     pack_str((char*)(req + 1), tableName);
@@ -1440,10 +1507,10 @@ int cli_alter_index(int session, char_t const* tableName, char_t const* fieldNam
     if (s == NULL) {
         return cli_bad_descriptor;
     }
-    int size = sizeof(cli_request) + 1 + (STRLEN(tableName)+STRLEN(fieldName)+2)*sizeof(char_t);
+    size_t size = sizeof(cli_request) + 1 + (STRLEN(tableName)+STRLEN(fieldName)+2)*sizeof(char_t);
     dbSmallBuffer<char> buf(size);
     cli_request* req = (cli_request*)buf.base();
-    req->length  = size;
+    req->length  = (int4)size;
     req->cmd     = cli_cmd_alter_index;
     req->stmt_id = 0;
     char* dst = (char*)(req + 1);
@@ -1463,7 +1530,7 @@ int cli_alter_index(int session, char_t const* tableName, char_t const* fieldNam
 }
 
 
-cli_error_handler cli_set_error_handler(int session, cli_error_handler new_handler)
+cli_error_handler cli_set_error_handler(int session, cli_error_handler new_handler, void* context)
 {
     return NULL;
 }
@@ -1478,6 +1545,20 @@ int cli_detach(int session, int detach_mode)
     return cli_not_implemented;
 }
 
+int cli_lock(int session)
+{
+    session_desc* s = sessions.get(session);
+    if (s == NULL) {
+        return cli_bad_descriptor;
+    }
+    cli_request req;
+    req.length = sizeof(req);
+    req.cmd = cli_cmd_lock;
+    req.stmt_id = 0;
+    req.pack();
+    return s->sock->write(&req, sizeof req) ? cli_ok : cli_network_error;
+} 
+
 void cli_free_memory(int, void* ptr)
 {
     free(ptr);
@@ -1487,6 +1568,26 @@ void cli_set_trace_function(cli_trace_function_t)
 { 
 }
 
+int cli_prepare_query(int session, char const* query)
+{
+    return cli_not_implemented;
+}
+
+int cli_execute_query(int statement, int for_update, void* record_struct, ...)
+{
+    return cli_not_implemented;
+}
+
+int cli_execute_query_ex(int statement, int for_update, void* record_struct, int n_params, int* param_types, void** param_value)
+{
+    return cli_not_implemented;
+}
+
+int cli_insert_struct(int session, char_t const* table_name, void* record_struct, cli_oid_t* oid)
+{
+    return cli_not_implemented;
+}
+
 int cli_get_field_size(cli_field_descriptor* fields, int field_no)
 {
     return sizeof_type[fields[field_no].type];
@@ -1505,3 +1606,28 @@ int cli_get_field_offset(cli_field_descriptor* fields, int field_no)
     return offs - size;
 }
 
+int cli_xml_export(int session, FILE* out, char_t const* const* tables, int n_tables, cli_export_method method)
+{
+    return cli_not_implemented;
+}
+
+int cli_xml_import(int session, FILE* in)
+{
+    return cli_not_implemented;
+}
+
+int cli_backup(int session, char_t const* file_name, int compactify)
+{
+    return cli_not_implemented;
+}
+
+int cli_schedule_backup(int session, char_t const* file_name, int period)
+{
+    return cli_not_implemented;
+}
+
+int cli_get_database_size(int session, cli_nat8_t* size)
+{
+    return cli_not_implemented;
+}
+
