@@ -11,30 +11,28 @@
 #ifndef __COMPILER_H__
 #define __COMPILER_H__
 
+#ifndef DO_NOT_USE_SETJMP
 #include <setjmp.h>
+#endif
+
+#ifdef USE_REGEX
+extern "C" { 
+#include <regex.h>
+};
+#endif
 
 BEGIN_GIGABASE_NAMESPACE
 
+#ifdef DO_NOT_USE_SETJMP
+struct CompileErrorException {};
+struct OutOfBoundsException {};
+#else
 #if defined(__osf__) || defined(__FreeBSD__)
 #define longjmp(b,s) _longjmp(b,s) // do not restore signal context
 #define setjmp(b)  _setjmp(b)
 #endif
-
-#define DEBUG_NONE  0
-#define DEBUG_CHECK 1
-#define DEBUG_TRACE 2
-
-#if GIGABASE_DEBUG == DEBUG_TRACE
-#define TRACE_MSG(x)  dbTrace x
-#else
-#define TRACE_MSG(x)
 #endif
 
-typedef void (*dbTraceFunctionPtr)(char_t* message);
-extern dbTraceFunctionPtr dbTraceFunction;
-extern GIGABASE_DLL_ENTRY void dbTrace(char_t* message, ...);
-
-
 enum dbvmCodes {
 #define DBVM(cop, type, n_operands, commutative) cop,
 #include "compiler.d"
@@ -43,7 +41,7 @@ dbvmLastCode
 
 
 #define IS_CONSTANT(c) \
-(unsigned(c) - dbvmLoadVarBool <= (unsigned)dbvmLoadVarStdString - dbvmLoadVarBool)
+(unsigned(c) - dbvmLoadVarBool <= (unsigned)dbvmLoadRectangleConstant - dbvmLoadVarBool)
 
 
 // 
@@ -130,6 +128,7 @@ enum tokens {
     tkn_select, 
     tkn_table,
     tkn_error,
+    tkn_match,
     tkn_last_token
 };
 
@@ -191,6 +190,13 @@ class GIGABASE_DLL_ENTRY dbExprNode {
             dbExprNode*         arg[3];   // the same as operand[0]
             void*               fptr;
         } func;
+
+#ifdef USE_REGEX
+        struct { 
+            dbExprNode*         opd;  
+            regex_t             re;
+        } regex;
+#endif
     };
 
     dbExprNode(dbExprNode* node);
@@ -294,6 +300,12 @@ class dbOrderByNode {
     dbExprNode*        expr;
     bool               ascent;  // true for ascent order, false for descent
     
+    dbFieldDescriptor* getField() {
+        return (field != NULL) ? field
+            : ((unsigned)expr->cop - dbvmLoadBool <= dbvmLoadRawBinary - dbvmLoadBool) 
+               ? expr->ref.field : NULL;
+    }
+
     ~dbOrderByNode() { 
         delete expr;
     }
@@ -310,7 +322,11 @@ class GIGABASE_DLL_ENTRY dbCompiler {
     friend class dbQueryElement;
   public:
     enum {
+#ifdef __SYMBIAN32__
+        maxStrLen    = 1024,
+#else
         maxStrLen    = 4096,
+#endif
         maxFreeVars  = 4
     };
 
@@ -332,7 +348,9 @@ class GIGABASE_DLL_ENTRY dbCompiler {
     void const*        varPtr;
     dbTableDescriptor* varRefTable;
 
+#ifndef DO_NOT_USE_SETJMP
     jmp_buf            abortCompilation;
+#endif
     static bool        initialized;
 
     void        compare(dbExprNode* expr, dbExprNode* list);
@@ -363,6 +381,7 @@ class GIGABASE_DLL_ENTRY dbCompiler {
     void        compileStartFollowPart(dbQuery& query);
 
     void        deleteNode(dbExprNode* node);
+    dbExprNode* rectangleConstant(dbExprNode* head);
 
     dbCompiler();
 };
@@ -385,8 +404,9 @@ class GIGABASE_DLL_ENTRY dbDatabaseThreadContext : public dbL2List {
     dbLockType pendingLock;
     dbDatabaseThreadContext* nextPending;
 
+#ifndef DO_NOT_USE_SETJMP
     jmp_buf  unwind;
-
+#endif
     dbDatabaseThreadContext() {
         concurrentId = 0;
         holdLock = dbNoLock;
@@ -421,18 +441,20 @@ class dbSynthesizedAttribute {
     enum ObjectStorageClass {
         osSelf,     // self object field
         osStack,    // object allocate on stack
-        osDynamic,  // object allocated by operstor new
+        osDynamic,  // object allocated by operator new
         osPage,     // reference to page in page pool
         osFree      // deallocated attribute
     };
     ObjectStorageClass osClass;
     union {
-        int sp;
+        size_t sp;
         struct {
             byte*                   addr;
             dbSynthesizedAttribute* next;
         } loc;
     } os;
+
+    dbSynthesizedAttribute() : osClass(osSelf) {}
 };
 
 class dbInheritedAttribute {
@@ -444,7 +466,7 @@ class dbInheritedAttribute {
     size_t             paramBase;
 
     enum {
-#ifdef _ARM
+#if defined(_ARM) || defined(__SYMBIAN32__)
         internalStackSize = 4*1024
 #else
         internalStackSize = 64*1024
@@ -459,7 +481,9 @@ class dbInheritedAttribute {
         int     index;
         int     sp;
         dbSynthesizedAttribute* dynChain;
+#ifndef DO_NOT_USE_SETJMP
         jmp_buf unwind;
+#endif
     } exists_iterator[dbCompiler::maxFreeVars];
 
     byte   stack[internalStackSize];
@@ -477,7 +501,11 @@ class dbInheritedAttribute {
         while (dynChain != ctx->dynChain) {
             free(*dynChain);
         }
+#ifdef DO_NOT_USE_SETJMP
+        throw OutOfBoundsException();
+#else
         longjmp(ctx->unwind, 1);
+#endif
     }
 
     void makeDynamic(dbSynthesizedAttribute& attr, void* p) {
@@ -500,7 +528,7 @@ class dbInheritedAttribute {
         }
     }
     void allocateString(dbSynthesizedAttribute& attr, char_t* str, size_t len) {
-        allocateString(attr, len);
+        allocateString(attr, (int)len);
         memcpy(attr.array.base, str, len*sizeof(char_t));
     }
     void allocateString(dbSynthesizedAttribute& attr, char_t* str) {
@@ -535,7 +563,7 @@ class dbInheritedAttribute {
         size_t size = rec->size;
         if (offs + size > dbPageSize) {
             byte* dst;
-            int start = DOALIGN(sp, 8);
+            size_t start = DOALIGN(sp, 8);
             if (start + size > sizeof(stack)) {
                 dst = dbMalloc(size);
                 makeDynamic(sattr, dst);
@@ -581,6 +609,31 @@ class dbInheritedAttribute {
     }
 };
 
+inline char_t* findWildcard(char_t* pattern, char_t* escape = NULL)
+{
+    if (escape == NULL) {
+        while (*pattern != dbMatchAnyOneChar &&
+               *pattern != dbMatchAnySubstring)
+        {
+            if (*pattern++ == '\0') {
+                return NULL;
+            }
+        }
+    } else {
+        char_t esc = *escape;
+        while (*pattern != dbMatchAnyOneChar &&
+               *pattern != dbMatchAnySubstring &&
+               *pattern != esc)
+        {
+            if (*pattern++ == '\0') {
+                return NULL;
+            }
+        }
+    }
+    return pattern;
+}
+
+
 END_GIGABASE_NAMESPACE
 
 #endif
