@@ -16,6 +16,82 @@
 
 BEGIN_GIGABASE_NAMESPACE
 
+size_t dbSelection::buildSelectionBitmapThreshold = 100;
+
+oid_t dbTableIterator::next()
+{
+    if (curr != 0) { 
+        dbRecord rec;
+        oid_t oid = curr;
+        while (true) { 
+            cursor->db->getHeader(rec, oid);
+            oid = rec.next;
+            if (oid != 0) { 
+                if (filter == NULL 
+                    || cursor->db->evaluateBoolean(filter, oid, cursor->table, cursor)) 
+                {
+                    return curr = oid;
+                }
+            } else { 
+                break;
+            }
+        }
+    }
+    return 0;
+}
+
+oid_t dbTableIterator::prev()
+{
+    if (curr != 0) { 
+        dbRecord rec;
+        oid_t oid = curr;
+        while (true) { 
+            cursor->db->getHeader(rec, oid);
+            oid = rec.prev;
+            if (oid != 0) { 
+                if (filter == NULL 
+                    || cursor->db->evaluateBoolean(filter, oid, cursor->table, cursor)) 
+                {
+                    return curr = oid;
+                }
+            } else { 
+                break;
+            }
+        }
+    }
+    return 0;
+}
+
+oid_t dbTableIterator::first()
+{
+    oid_t oid = cursor->table->firstRow;
+    while (oid != 0
+           && filter != NULL 
+           && !cursor->db->evaluateBoolean(filter, oid, cursor->table, cursor))
+    {
+        dbRecord rec;
+        cursor->db->getHeader(rec, oid);
+        oid = rec.next;
+    }
+    return curr = oid;
+}
+
+    
+oid_t dbTableIterator::last()
+{
+    oid_t oid = cursor->table->lastRow;
+    while (oid != 0
+           && filter != NULL 
+           && !cursor->db->evaluateBoolean(filter, oid, cursor->table, cursor))
+    {
+        dbRecord rec;
+        cursor->db->getHeader(rec, oid);
+        oid = rec.prev;
+    }
+    return curr = oid;
+}
+
+
 void dbSelection::reset()
 {
     while (first.next != &first) {
@@ -34,7 +110,7 @@ void dbSelection::reverse()
         segment* next = seg->next;
         seg->next = seg->prev;
         seg->prev = next;
-        for (int l = 0, r = seg->nRows-1; l < r; l++, r--) {
+        for (int l = 0, r = (int)seg->nRows-1; l < r; l++, r--) {
             oid_t oid = seg->rows[l];
             seg->rows[l] = seg->rows[r];
             seg->rows[r] = oid;
@@ -43,18 +119,26 @@ void dbSelection::reverse()
     } while (seg != &first);
 }
 
-void dbSelection::truncate(size_t from, size_t length)
+void dbSelection::truncate(cardinality_t from, cardinality_t length)
 {
+    if (from == 0 && length >= nRows) { 
+        // do nothing
+        return;
+    }
     segment* src = &first;
     bool empty = true;
-    do { 
-        if (from < src->nRows) { 
-            empty = false;
-            break;
-        }
-        from -= src->nRows;
-    } while ((src = src->next) != &first);
-
+    if (from < nRows) { 
+        do { 
+            if (from < src->nRows) { 
+                empty = false;
+                break;
+            }
+            from -= src->nRows;
+        } while ((src = src->next) != &first);
+    }
+    if (from + length > nRows) { 
+        length = nRows - from;
+    }
     nRows = 0;
     segment* dst = &first;
     size_t pos = 0;
@@ -92,7 +176,7 @@ void dbSelection::truncate(size_t from, size_t length)
     }
 }
 
-int dbSelection::compare(dbRecord* a, dbRecord* b, dbOrderByNode* order)
+int dbSelection::compare(oid_t o1, dbRecord* a, oid_t o2, dbRecord* b, dbOrderByNode* order)
 {
     byte* p = (byte*)a;
     byte* q = (byte*)b;
@@ -107,7 +191,9 @@ int dbSelection::compare(dbRecord* a, dbRecord* b, dbOrderByNode* order)
             iattr1.db = iattr2.db = db;
             iattr1.table = iattr2.table = order->table;
             iattr1.record = sattr1.base = p;
+            iattr1.oid = o1;
             iattr2.record = sattr2.base = q;
+            iattr2.oid = o2;
             db->execute(order->expr, iattr1, sattr1);
             db->execute(order->expr, iattr2, sattr2);
             switch (order->expr->type) { 
@@ -199,24 +285,25 @@ struct dbSortContext {
 
 inline db_int8 packStrKey(byte* p, bool caseInsensitive)
 {
+    char_t cnvBuf[8/sizeof(char_t) + 1];
+    if (caseInsensitive) { 
+        size_t i;
+        for (i = 0; i < 8/sizeof(char_t) && *((char_t*)p + i) != 0; i++) {
+            cnvBuf[i] = TOLOWER(*((char_t*)p + i));
+        }
+        cnvBuf[i] = 0;
+        p = (byte*)cnvBuf;
+    }            
 #ifdef USE_LOCALE_SETTINGS
     char_t buf[8/sizeof(char_t)];
     STRXFRM(buf, (char_t*)p, itemsof(buf));
     p = (byte*)buf;
 #endif
     db_int8 pkey = 0;
-    if (caseInsensitive) { 
-        for (size_t i = 0; i < 8/sizeof(char_t) && *((char_t*)p + i) != 0; i++) {
-            char_t ch = TOLOWER(*((char_t*)p + i));
-            pkey |= ((db_int8)ch & ((1 << sizeof(char_t)*8)-1))
-                                       << ((8/sizeof(char_t)-1-i)*8*sizeof(char_t));
-        }
-    } else { 
-        for (size_t i = 0; i < 8/sizeof(char_t) && *((char_t*)p + i) != 0; i++) {
-            char_t ch = *((char_t*)p + i);
-            pkey |= ((db_int8)ch & ((1 << sizeof(char_t)*8)-1))
-                                       << ((8/sizeof(char_t)-1-i)*8*sizeof(char_t));
-        }
+    for (size_t i = 0; i < 8/sizeof(char_t) && *((char_t*)p + i) != 0; i++) {
+        char_t ch = *((char_t*)p + i);
+        pkey |= ((db_int8)ch & ((1 << sizeof(char_t)*8)-1))
+            << ((8/sizeof(char_t)-1-i)*8*sizeof(char_t));
     }
     // As far as signed comparison is used for packed key,
     // and strcmp compares characters as unsign, we should make this
@@ -286,8 +373,8 @@ int __cdecl dbSelection::exactKeyCmp(void const* a, void const* b)
 {
     dbGetTie ta, tb;
     dbSortContext* ctx = sortThreadContext.get();
-    return compare(ctx->db->getRow(ta, ((dbSortRecord*)a)->oid),
-                   ctx->db->getRow(tb, ((dbSortRecord*)b)->oid),
+    return compare(((dbSortRecord*)a)->oid, ctx->db->getRow(ta, ((dbSortRecord*)a)->oid),
+                   ((dbSortRecord*)b)->oid, ctx->db->getRow(tb, ((dbSortRecord*)b)->oid),
                    ctx->order);
 }
 
@@ -303,7 +390,7 @@ void dbSelection::toArray(oid_t* oids) const
 {
     segment const* seg = &first;
     do {
-        for (int i = 0, n = seg->nRows; i < n; i++) {
+        for (int i = 0, n = (int)seg->nRows; i < n; i++) {
             *oids++ = seg->rows[i];
         }
     } while ((seg = seg->next) != &first);
@@ -314,56 +401,106 @@ static int __cdecl compareOids(void const* a, void const* b)
     return *(oid_t*)a < *(oid_t*)b ? -1 : *(oid_t*)a == *(oid_t*)b ? 0 : 1;
 }
 
-void dbSelection::merge(dbSelection& selection)
-{
-    int n1 = nRows, n2 = selection.nRows;
-    TRACE_MSG((STRLITERAL("Performing indexed merge of %d and %d rows\n"), n1, n2));
-    dbSmallBuffer<oid_t> buf2(n2);
-    dbSmallBuffer<oid_t> buf1(n1);
-    oid_t* refs1 = buf1.base();
-    oid_t* refs2 = buf2.base();
-    toArray(refs1);
-    selection.toArray(refs2);
-    qsort(refs1, nRows, sizeof(oid_t), &compareOids);
-    qsort(refs2, selection.nRows, sizeof(oid_t), &compareOids);
-    int i1 = 0, i2 = 0, i = 0, j = 0;
-    segment* seg = &first;
 
-    while (true) {
-        if (i1 == n1 || i2 == n2) {
-            break;
-        }
-        if (refs1[i1] > refs2[i2]) {
-            i2 += 1;
-        } else if (refs1[i1] < refs2[i2]) {
-            i1 += 1;
-        } else {
-            if (j == (int)seg->nRows) {
-                seg = seg->next;
-                j = 0;
+void dbSelection::allocateBitmap(dbDatabase* db) 
+{
+    size_t size = (size_t)((db->currIndexSize + 31) / 32);
+    if (size > bitmapSize) {
+        delete[] bitmap;
+        bitmap = new int4[size];
+        bitmapSize = size;
+    }
+    memset(bitmap, 0, size*4);
+}
+
+void dbSelection::deallocateBitmap() 
+{
+    if (bitmap != NULL) { 
+        delete[] bitmap;
+        bitmap = NULL;
+        bitmapSize = 0;
+    }
+}
+
+void dbSelection::merge(dbDatabase* db, dbSelection& selection)
+{
+    size_t n1 = nRows, n2 = selection.nRows;
+    TRACE_MSG((STRLITERAL("Performing indexed merge of %d and %d rows\n"), n1, n2));
+    segment* dst = &first;
+    size_t n = 0, j = 0;
+
+    if (n1 > buildSelectionBitmapThreshold || n2 > buildSelectionBitmapThreshold) { 
+        allocateBitmap(db);
+        int4* bm = bitmap;
+        segment const* seg = &selection.first;        
+        do {
+            for (size_t i = 0, sn = seg->nRows; i < sn; i++) {
+                oid_t o = seg->rows[i];
+                bm[(size_t)o >> 5] |= 1 << ((int)o & 31);
+            }
+        } while ((seg = seg->next) != &selection.first);
+
+        seg = &first;
+        do {
+            for (size_t i = 0, sn = seg->nRows; i < sn; i++) {
+                oid_t o = seg->rows[i];
+                if (bm[(size_t)o >> 5] & (1 << ((int)o & 31))) {
+                    if (j == dst->nRows) { 
+                        dst = dst->next;
+                        j = 0;
+                    }                    
+                    dst->rows[j++] = o;
+                    n += 1;
+                }
+            }
+        } while ((seg = seg->next) != &first);     
+    } else { 
+        dbSmallBuffer<oid_t> buf2(n2);
+        dbSmallBuffer<oid_t> buf1(n1);
+        oid_t* refs1 = buf1.base();
+        oid_t* refs2 = buf2.base();
+        toArray(refs1);
+        selection.toArray(refs2);
+        qsort(refs1, nRows, sizeof(oid_t), &compareOids);
+        qsort(refs2, selection.nRows, sizeof(oid_t), &compareOids);
+        size_t i1 = 0, i2 = 0;
+        
+        while (true) {
+            if (i1 == n1 || i2 == n2) {
+                break;
+            }
+            if (refs1[i1] > refs2[i2]) {
+                i2 += 1;
+            } else if (refs1[i1] < refs2[i2]) {
+                i1 += 1;
+            } else {
+                if (j == dst->nRows) {
+                    dst = dst->next;
+                    j = 0;
+                }
+                n += 1;
+                dst->rows[j++] = refs1[i1];
+                i1 += 1;
+                i2 += 1;
             }
-            i += 1;
-            seg->rows[j++] = refs1[i1];
-            i1 += 1;
-            i2 += 1;
         }
     }
-    seg->nRows = j;
-    nRows = i;
-    segment* next = seg->next;
-    first.prev = seg;
-    seg->next = &first;
+    dst->nRows = j;
+    nRows = n;
+    segment* next = dst->next;
+    first.prev = dst;
+    dst->next = &first;
     while (next != &first) {
-        seg = next;
-        next = seg->next;
-        delete seg;
+        dst = next;
+        next = dst->next;
+        delete dst;
     }
 }
 
 
 void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitive, dbSortResult* sortResult)
 {
-    int i = 0, j, k, n = nRows;
+    int i = 0, j, k, n = (int)nRows;
     dbSortRecord* keys = new dbSortRecord[n];
     char* rawKeys = NULL;
     segment* seg = &first;
@@ -376,7 +513,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
         switch (order->expr->type) { 
           case tpReal: 
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     db->evaluate(order->expr, seg->rows[j], order->table, result);
                     keys[i].u.realKey = result.fvalue;
                     keys[i].oid = seg->rows[j];
@@ -386,7 +523,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
             break;
           case tpInteger:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     db->evaluate(order->expr, seg->rows[j], order->table, result);
                     keys[i].u.longKey = result.ivalue;
                     keys[i].oid = seg->rows[j];
@@ -396,7 +533,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
             break;
           case tpBoolean:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     db->evaluate(order->expr, seg->rows[j], order->table, result);
                     keys[i].u.intKey = result.bvalue;
                     keys[i].oid = seg->rows[j];
@@ -406,7 +543,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
             break;
           case tpReference:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     db->evaluate(order->expr, seg->rows[j], order->table, result);
                     keys[i].u.longKey = result.oid;
                     keys[i].oid = seg->rows[j];
@@ -417,9 +554,9 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
           case tpString:
             if (sortResult != NULL) { 
                 do {
-                    for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                    for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                         char_t buf[dbBtreePage::dbMaxKeyLen];
-                        int len = db->evaluateString(order->expr, seg->rows[j], order->table, buf, itemsof(buf));
+                        int len = (int)db->evaluateString(order->expr, seg->rows[j], order->table, buf, itemsof(buf));
                         assert(len <= dbBtreePage::dbMaxKeyLen);
                         keys[i].u.strKey = sortResult->strBuf.put(buf, len);
                         if (caseInsensitive) { 
@@ -431,7 +568,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
                 qsort(keys, n, sizeof(dbSortRecord), order->ascent ? cmpStrKey : cmpStrKeyDesc);
             } else { 
                 do {
-                    for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                    for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                         char_t buf[8/sizeof(char_t)];
                         db->evaluateString(order->expr, seg->rows[j], order->table, buf, itemsof(buf));
                         keys[i].u.longKey = packStrKey((byte*)buf, caseInsensitive);
@@ -456,7 +593,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
           case dbField::tpBool:
           case dbField::tpInt1:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     p = (byte*)db->getRow(tie, seg->rows[j]);
                     keys[i].u.intKey = *(int1*)(p + offs);
                     keys[i].oid = seg->rows[j];
@@ -466,7 +603,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
             break;
           case dbField::tpInt2:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     p = (byte*)db->getRow(tie, seg->rows[j]);
                     keys[i].u.intKey = *(int2*)(p + offs);
                     keys[i].oid = seg->rows[j];
@@ -476,7 +613,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
             break;
           case dbField::tpReal4:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     p = (byte*)db->getRow(tie, seg->rows[j]);
                     keys[i].u.realKey = *(float*)(p + offs);
                     keys[i].oid = seg->rows[j];
@@ -487,7 +624,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
           case dbField::tpInt4:
           case dbField::tpArray:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     p = (byte*)db->getRow(tie, seg->rows[j]);
                     keys[i].u.intKey = *(int4*)(p + offs);
                     keys[i].oid = seg->rows[j];
@@ -497,7 +634,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
             break;
           case dbField::tpInt8:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     p = (byte*)db->getRow(tie, seg->rows[j]);
                     keys[i].u.longKey = *(db_int8*)(p + offs);
                     keys[i].oid = seg->rows[j];
@@ -507,17 +644,17 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
             break;
           case dbField::tpReference:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
-                p = (byte*)db->getRow(tie, seg->rows[j]);
-                keys[i].u.longKey = *(oid_t*)(p + offs);
-                keys[i].oid = seg->rows[j];
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
+                    p = (byte*)db->getRow(tie, seg->rows[j]);
+                    keys[i].u.longKey = *(oid_t*)(p + offs);
+                    keys[i].oid = seg->rows[j];
                 }
             } while ((seg = seg->next) != &first);
             qsort(keys, n, sizeof(dbSortRecord), order->ascent ? cmpLongKey : cmpLongKeyDesc);
             break;
           case dbField::tpReal8:
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     p = (byte*)db->getRow(tie, seg->rows[j]);
                     keys[i].u.realKey = *(real8*)(p + offs);
                     keys[i].oid = seg->rows[j];
@@ -528,7 +665,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
           case dbField::tpString:
             if (sortResult != NULL) { 
                 do {
-                    for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                    for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                         byte* p = (byte*)db->getRow(tie, seg->rows[j]);
                         keys[i].u.strKey = sortResult->strBuf.put((char_t*)(p + ((dbVarying*)(p + offs))->offs), 
                                                                   ((dbVarying*)(p + offs))->size-1);
@@ -541,7 +678,7 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
                 qsort(keys, n, sizeof(dbSortRecord), order->ascent ? cmpStrKey : cmpStrKeyDesc);
             } else { 
                 do {
-                    for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                    for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                         byte* p = (byte*)db->getRow(tie, seg->rows[j]);
                         keys[i].u.longKey = packStrKey(p + ((dbVarying*)(p + offs))->offs, caseInsensitive);
                         keys[i].oid = seg->rows[j];
@@ -553,14 +690,14 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
             break;
           case dbField::tpRawBinary:
           { 
-            int rawSize = order->field->dbsSize;
+            int rawSize = (int)order->field->dbsSize;
             rawKeys = new char[n*rawSize];
             dbSortContext ctx;
             ctx.db = db;
             ctx.order = order;
             sortThreadContext.set(&ctx);
             do {
-                for (j = 0, k = seg->nRows; j < k; j++, i++) {
+                for (j = 0, k = (int)seg->nRows; j < k; j++, i++) {
                     p = (byte*)db->getRow(tie, seg->rows[j]);
                     keys[i].oid = seg->rows[j];
                     keys[i].u.rawKey = rawKeys + i*rawSize;
@@ -602,10 +739,10 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
         sortResult->rawKeys = rawKeys;        
     } else { 
         if (n != 0) {
-            for (i = 0, j = 0, seg = &first, k = seg->nRows; i < n; i++, j++) {
+            for (i = 0, j = 0, seg = &first, k = (int)seg->nRows; i < n; i++, j++) {
                 if (j == k) {
                     seg = seg->next;
-                    k = seg->nRows;
+                    k = (int)seg->nRows;
                     j = 0;
                 }
                 seg->rows[j] = keys[i].oid;
@@ -615,9 +752,61 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order, bool caseInsensitiv
         delete[] rawKeys;
     }
 }
+    
+dbAnyCursor::dbAnyCursor(dbTableDescriptor& aTable, dbCursorType aType, byte* rec)
+: table(&aTable),type(aType),defaultType(aType),
+  allRecords(false),currId(0),record(rec)
+{
+    limit = dbDefaultSelectionLimit;
+    prefetch = rec != NULL;
+    removed = false;
+    eliminateDuplicates = false;
+    checkForDuplicatedIsEnabled = true;
+    db = aTable.db;
+    paramBase = NULL;
+    stmtLimitLen = dbDefaultSelectionLimit;
+    stmtLimitStart = 0;
+    nSkipped = 0; 
+    iterator = NULL;
+}
+
+dbAnyCursor::dbAnyCursor(dbCursorType aType) 
+: table(NULL),type(aType),defaultType(aType),
+  allRecords(false),currId(0),record(NULL)
+{
+    limit = dbDefaultSelectionLimit;
+    prefetch = false;
+    removed = false;
+    eliminateDuplicates = false;
+    checkForDuplicatedIsEnabled = true;
+    db = NULL;
+    paramBase = NULL;
+    stmtLimitLen = dbDefaultSelectionLimit;
+    stmtLimitStart = 0;
+    nSkipped = 0;
+    iterator = NULL;
+}
+
+cardinality_t dbAnyCursor::select(dbQuery& query, dbCursorType aType, void* paramStruct) 
+{
+    paramBase = paramStruct;
+    type = aType;
+    reset();
+    db->select(this, query);
+    paramBase = NULL;
+    if (gotoFirst() && prefetch) {
+        fetch();
+    }
+    if (aType == dbCursorDetached) { 
+        unlink();    
+        db->commit();
+    }
+    return selection.nRows;
+}
 
 oid_t* dbAnyCursor::toArrayOfOid(oid_t* arr) const
 { 
+    assert(iterator == NULL); // do not use incremental cursor if you need to get array of OIDs
     if (arr == NULL) { 
         arr = new oid_t[selection.nRows];
     }
@@ -649,24 +838,38 @@ void dbAnyCursor::setCurrent(dbAnyReference const& ref)
     }
 }
 
-int dbAnyCursor::selectByKey(char_t const* key, void const* value)
+cardinality_t dbAnyCursor::selectByKey(char_t const* key, void const* value)
 {
     dbFieldDescriptor* field = table->find(key);
     assert(field != NULL);
+    return selectByKey(field, value);
+}
+
+cardinality_t dbAnyCursor::selectByKey(dbFieldDescriptor* field, void const* value)
+{
     assert(field->hashTable != 0 || field->bTree != 0);
     reset();
     db->beginTransaction(type == dbCursorForUpdate ? dbUpdateLock : dbSharedLock);
     db->threadContext.get()->cursors.link(this);
     dbSearchContext sc;
     sc.db = db;
+    sc.probes = 0;
     sc.ascent = true;
     sc.offs = field->dbsOffs;
     sc.cursor = this;
     sc.tmpKeys = false;
     sc.condition = NULL;
+    sc.prefixLength = 0;
+    sc.spatialSearch = false;
+    sc.arraySearch = false;
     sc.firstKey = sc.lastKey = (char_t*)value;
     sc.firstKeyInclusion = sc.lastKeyInclusion = true;
-    dbBtree::find(db, field->bTree, sc, field->comparator);
+    if (type == dbCursorIncremental) { 
+        btreeIterator.init(db, field->bTree, sc, field->comparator);
+        iterator = &btreeIterator;
+    } else {
+        dbBtree::find(db, field->bTree, sc, field->comparator);
+    }
     if (gotoFirst() && prefetch) {
         fetch();
     }
@@ -709,26 +912,41 @@ bool dbAnyCursor::skip(int n) {
     return true;
 }
 
-int dbAnyCursor::selectByKeyRange(char_t const* key, void const* minValue, 
+cardinality_t dbAnyCursor::selectByKeyRange(char_t const* key, void const* minValue, 
                                   void const* maxValue, bool ascent)
 {
     dbFieldDescriptor* field = table->find(key);
     assert(field != NULL);
+    return selectByKeyRange(field, minValue, maxValue, ascent);
+}
+
+cardinality_t dbAnyCursor::selectByKeyRange(dbFieldDescriptor* field, void const* minValue, 
+                                  void const* maxValue, bool ascent)
+{
     assert(field->bTree != 0);
     reset();
     db->beginTransaction(type == dbCursorForUpdate ? dbUpdateLock : dbSharedLock);
     db->threadContext.get()->cursors.link(this);
     dbSearchContext sc;
     sc.db = db;
+    sc.probes = 0;
     sc.offs = field->dbsOffs;
     sc.cursor = this;
     sc.tmpKeys = false;
     sc.ascent = ascent;
     sc.condition = NULL;
+    sc.prefixLength = 0;
     sc.firstKey = (char_t*)minValue;
     sc.lastKey = (char_t*)maxValue;
     sc.firstKeyInclusion = sc.lastKeyInclusion = true;
-    dbBtree::find(db, field->bTree, sc, field->comparator);
+    sc.spatialSearch = false;
+    sc.arraySearch = false;
+    if (type == dbCursorIncremental) { 
+        btreeIterator.init(db, field->bTree, sc, field->comparator);
+        iterator = &btreeIterator;
+    } else { 
+        dbBtree::find(db, field->bTree, sc, field->comparator);
+    }
     if (gotoFirst() && prefetch) {
         fetch();
     }
@@ -738,7 +956,13 @@ int dbAnyCursor::selectByKeyRange(char_t const* key, void const* minValue,
 void dbAnyCursor::remove()
 {
     oid_t removedId = currId;
-    assert(type == dbCursorForUpdate && removedId != 0);
+    lastRecordWasDeleted = false;
+    if (type != dbCursorForUpdate) {
+        db->handleError(dbDatabase::CursorError, "Readonly cursor");
+    }
+    if (removedId == 0) {
+        db->handleError(dbDatabase::CursorError, "No current record");
+    }
     if (allRecords) {
         dbRecord rec;
         db->getHeader(rec, removedId);
@@ -749,6 +973,7 @@ void dbAnyCursor::remove()
                 currId = rec.next;
             }
         } else {
+            lastRecordWasDeleted = true;
             if (removedId == firstId) {
                 firstId = lastId = currId = 0;
             } else {
@@ -766,6 +991,7 @@ void dbAnyCursor::remove()
                     selection.curr = next;
                     selection.pos = 0;
                 } else {
+                    lastRecordWasDeleted = true;
                     selection.curr = next->prev;
                     selection.pos = selection.curr->nRows-1;
                 }
@@ -785,24 +1011,31 @@ void dbAnyCursor::remove()
             currId = 0;
         }
     }
+    byte* saveRecord = record;
+    record = NULL;
     db->remove(table, removedId);
-    if (currId != 0) {
+    record = saveRecord;
+    removed = true;
+    if (currId != 0 && prefetch) {
         fetch();
     }
-    removed = true;
 }
 
 
 void dbAnyCursor::removeAllSelected()
 {
-    assert(type == dbCursorForUpdate);
+    if (type != dbCursorForUpdate) {
+        db->handleError(dbDatabase::CursorError, "Readonly cursor");
+    }
+    byte* saveRecord = record;
+    record = NULL;
     if (allRecords) {
         removeAll();
     } else if (selection.nRows != 0) {
         dbSelection::segment* curr = &selection.first;
         currId = 0;
         do {
-            for (int i = 0, n = curr->nRows; i < n; i++) {
+            for (int i = 0, n = (int)curr->nRows; i < n; i++) {
                 db->remove(table, curr->rows[i]);
             }
         } while ((curr = curr->next) != &selection.first);
@@ -811,11 +1044,19 @@ void dbAnyCursor::removeAllSelected()
         db->remove(table, currId);
         currId = 0;
     }
+    record = saveRecord;
 }
 
 bool dbAnyCursor::isLast() const
 { 
-    if (allRecords) { 
+    if (iterator != NULL) { 
+        if (currId != 0) { 
+            if (!iterator->next()) {
+                return true;
+            }
+            iterator->prev();
+        }
+    } else if (allRecords) { 
         if (currId != 0) { 
             dbRecord rec;
             db->getHeader(rec, currId);
@@ -831,7 +1072,14 @@ bool dbAnyCursor::isLast() const
 
 bool dbAnyCursor::isFirst() const
 { 
-    if (allRecords) { 
+    if (iterator != NULL) { 
+        if (currId != 0) { 
+            if (!iterator->prev()) {
+                return true;
+            }
+            iterator->next();
+        }
+    } else if (allRecords) { 
         if (currId != 0) { 
             dbRecord rec;
             db->getHeader(rec, currId);
@@ -844,8 +1092,167 @@ bool dbAnyCursor::isFirst() const
 }
 
 
+bool dbAnyCursor::hasNext() const
+{ 
+    if (iterator != NULL) { 
+        if (currId != 0 && iterator->next()) {
+            iterator->prev();
+            return true;
+        }
+    } else if (allRecords) {
+        if (currId != 0) {
+            dbRecord rec;
+            db->getHeader(rec, currId);
+            return rec.next != 0;
+        }
+    } else if (selection.curr != NULL 
+               && (selection.pos+1 < selection.curr->nRows || selection.curr->next != &selection.first))
+    {
+        return true;
+    }
+    return false;
+}
+
+byte* dbAnyCursor::fetchNext()
+{
+    byte* result = NULL;
+    if (type == dbCursorDetached) { 
+        db->beginTransaction(dbSharedLock);
+        db->threadContext.get()->cursors.link(this);
+        assert(!removed);
+        while (gotoNext()) {
+            if (db->isValidOid(currId)) { 
+                fetch();
+                result = record;
+                break;
+            }
+        }
+        unlink();    
+        db->commit();
+    } else { 
+        if (!removed) {
+            if (gotoNext()) {
+                fetch();
+                result = record;
+            }
+        } else { 
+            removed = false; 
+            if (currId != 0 && !lastRecordWasDeleted) { 
+                if (!prefetch) { 
+                    fetch();
+                }
+                result = record;
+            }
+        }
+    }
+    return result;  
+}
+
+byte* dbAnyCursor::fetchPrev()
+{
+    if (type == dbCursorDetached) { 
+        db->beginTransaction(dbSharedLock);
+        db->threadContext.get()->cursors.link(this);
+        assert(!removed);
+        byte* result = NULL;
+        while (gotoPrev()) {            
+            if (db->isValidOid(currId)) { 
+                fetch();
+                result = record;
+                break;
+            }
+        }
+        unlink();    
+        db->commit();
+        return result;
+    } else { 
+        if (removed) {
+            removed = false; 
+            if (lastRecordWasDeleted) { 
+                if (currId != 0) { 
+                    if (!prefetch) { 
+                        fetch();
+                    }
+                    return record;
+                }
+                return NULL;
+            }
+        }
+        if (gotoPrev()) {
+            fetch();
+            return record;
+        }
+        return NULL;
+    }
+}
+
+byte* dbAnyCursor::fetchFirst()
+{
+    byte* result = NULL;
+
+    if (type == dbCursorDetached) { 
+        db->beginTransaction(dbSharedLock);
+        db->threadContext.get()->cursors.link(this);
+
+        if (gotoFirst()) {
+            do { 
+                if (db->isValidOid(currId)) {                 
+                    fetch();
+                    result = record;
+                    break;
+                }
+            } while (gotoNext());
+        }
+        unlink();    
+        db->commit();
+    } else { 
+        if (gotoFirst()) {
+            fetch();
+            result = record;
+        }
+    }
+    return result;
+}
+
+byte* dbAnyCursor::fetchLast()
+{
+    byte* result = NULL;
+
+    if (type == dbCursorDetached) { 
+        db->beginTransaction(dbSharedLock);
+        db->threadContext.get()->cursors.link(this);
+
+        if (gotoLast()) {
+            do { 
+                if (db->isValidOid(currId)) {                 
+                    fetch();
+                    result = record;
+                    break;
+                }
+            } while (gotoPrev());
+        }
+        unlink();    
+        db->commit();
+    } else { 
+        if (gotoLast()) {
+            fetch();
+            result = record;
+        }
+    }
+    return result;
+}
+
+
 bool dbAnyCursor::gotoNext()
 {
+    if (iterator != NULL) { 
+        oid_t next = iterator->next();
+        if (next != 0) { 
+            currId = next;
+            return true;
+        }
+        return false;
+    } 
     removed = false;
     if (allRecords) {
         if (currId != 0) {
@@ -873,6 +1280,14 @@ bool dbAnyCursor::gotoNext()
 
 bool dbAnyCursor::gotoPrev()
 {
+    if (iterator != NULL) { 
+        oid_t prev = iterator->prev();
+        if (prev != 0) { 
+            currId = prev;
+            return true;
+        }
+        return false;
+    } 
     removed = false;
     if (allRecords) {
         if (currId != 0) {
@@ -898,8 +1313,35 @@ bool dbAnyCursor::gotoPrev()
 }
 
 
+bool dbAnyCursor::moveNext() 
+{
+    if (!removed) { 
+        return gotoNext(); 
+    } else {
+        removed = false;
+        return !lastRecordWasDeleted;
+    }
+}
+    
+bool dbAnyCursor::movePrev() 
+{
+    if (!removed) { 
+        return gotoPrev(); 
+    } else {
+        removed = false;
+        return lastRecordWasDeleted ? (currId != 0) : gotoPrev();
+    }
+}
+ 
 bool dbAnyCursor::gotoFirst()
 {
+    if (iterator != NULL) { 
+        if ((currId = iterator->first()) != 0) { 
+            selection.nRows = 1;
+            return true;
+        }
+        return false;
+    } 
     removed = false;
     if (allRecords) {
         currId = firstId;
@@ -918,6 +1360,9 @@ bool dbAnyCursor::gotoFirst()
 
 bool dbAnyCursor::gotoLast()
 {
+    if (iterator != NULL) { 
+        return (currId = iterator->last()) != 0;
+    } 
     removed = false;
     if (allRecords) {
         currId = lastId;
@@ -934,26 +1379,25 @@ bool dbAnyCursor::gotoLast()
     }
 }
 
-#define BUILD_BITMAP_THRESHOLD 100
-
 bool dbAnyCursor::isInSelection(oid_t oid) 
 {
+    assert(iterator == NULL);
     if (eliminateDuplicates) { 
         return isMarked(oid);
-    } else if (selection.nRows > BUILD_BITMAP_THRESHOLD) {
+    } else if (selection.nRows > dbSelection::buildSelectionBitmapThreshold) {
         checkForDuplicates();
         dbSelection::segment* curr = &selection.first;
         do {
-            for (int i = 0, n = curr->nRows; i < n; i++) {
+            for (int i = 0, n = (int)curr->nRows; i < n; i++) {
                 oid_t o = curr->rows[i];
-                bitmap[o >> 5] |= 1 << (o & 31);
+                selection.bitmap[(size_t)o >> 5] |= 1 << ((int)o & 31);
             }
         } while ((curr = curr->next) != &selection.first);
         return isMarked(oid);
     } else { 
         dbSelection::segment* curr = &selection.first;
         do {
-            for (int i = 0, n = curr->nRows; i < n; i++) {
+            for (int i = 0, n = (int)curr->nRows; i < n; i++) {
                 if (curr->rows[i] == oid) { 
                     return true;
                 }
@@ -982,10 +1426,12 @@ void dbAnyCursor::reset()
     stmtLimitLen = dbDefaultSelectionLimit;
     stmtLimitStart = 0;
     nSkipped = 0;
+    iterator = NULL;
 }
 
 void dbAnyCursor::freeze()
 {
+    assert(type != dbCursorIncremental && type != dbCursorDetached);
     unlink();    
     tie.reset();
 }
@@ -1002,25 +1448,8 @@ void dbAnyCursor::unfreeze()
 dbAnyCursor::~dbAnyCursor()
 {
     selection.reset();
-    delete[] bitmap;
 }
 
-
-void dbAnyCursor::allocateBitmap() 
-{
-    if (!eliminateDuplicates && limit > 1) {
-        eliminateDuplicates = true;
-        size_t size = (db->currIndexSize + 31) / 32;
-        if (size > bitmapSize) {
-            delete[] bitmap;
-            bitmap = new int4[size];
-            bitmapSize = size;
-        }
-        memset(bitmap, 0, size*4);
-    }
-}
-
-
 void dbParallelQueryContext::search(int i)
 {
     int nThreads = db->parThreads;
