@@ -19,8 +19,8 @@ BEGIN_GIGABASE_NAMESPACE
 
 dbException::dbException(int p_err_code, const char* p_msg, int p_arg)
 : err_code (p_err_code),
-    msg (NULL),
-    arg (p_arg)
+  msg (NULL),
+  arg (p_arg)
 {
     if (p_msg != NULL) {
         msg = new char[strlen(p_msg)+1];
@@ -32,6 +32,7 @@ dbException::dbException(dbException const& ex)
 {
     err_code = ex.err_code;
     arg = ex.arg;
+    msg = ex.msg;
     if (ex.msg != NULL) {
         msg = new char[strlen(ex.msg)+1];
         strcpy(msg, ex.msg);
@@ -40,11 +41,17 @@ dbException::dbException(dbException const& ex)
     }
 }
 
-dbException::~dbException() 
+dbException::~dbException() throw()
 {
     delete[] msg;
 }
 
+const char* dbException::what() const throw()
+{
+    return getMsg();
+}
+
+
 END_GIGABASE_NAMESPACE
 
 #endif
