@@ -24,7 +24,7 @@ byte* dbPagePool::find(offs_t addr, int state)
     dbPageHeader* ph;
     assert(((int)addr & (dbPageSize-1)) == 0);
 
-    int hashCode = (unsigned(addr) >> dbPageBits) & hashBits;
+    int hashCode = (int)((unsigned(addr) >> dbPageBits) & hashBits);
     int i, rc;
     mutex.lock();
     for (i = hashTable[hashCode]; i != 0; i = ph->collisionChain) {
@@ -64,7 +64,7 @@ byte* dbPagePool::find(offs_t addr, int state)
                 (state & dbPageHeader::psDirty))
             {
                 dirtyPages[nDirtyPages] = ph;
-                ph->writeQueueIndex = nDirtyPages++;
+                ph->writeQueueIndex = (int4)(nDirtyPages++);
             } 
 #ifdef PROTECT_PAGE_POOL
             if ((state & dbPageHeader::psDirty)) { 
@@ -99,7 +99,7 @@ byte* dbPagePool::find(offs_t addr, int state)
                 fileSize = ph->offs + dbPageSize;
             }
         }
-        unsigned h = (unsigned(ph->offs) >> dbPageBits) & hashBits;
+        unsigned h = (unsigned)((unsigned(ph->offs) >> dbPageBits) & hashBits);
         int* np;
         for (np = &hashTable[h]; *np != i; np = &pages[*np].collisionChain);
         *np = ph->collisionChain;
@@ -122,7 +122,7 @@ byte* dbPagePool::find(offs_t addr, int state)
 
     if (state & dbPageHeader::psDirty) {
         dirtyPages[nDirtyPages] = ph;
-        ph->writeQueueIndex = nDirtyPages++;
+        ph->writeQueueIndex = (int4)(nDirtyPages++);
         ph->state |= dbPageHeader::psDirty;
     }
 
@@ -160,12 +160,12 @@ byte* dbPagePool::find(offs_t addr, int state)
     return p;
 }
 
-void dbPagePool::copy(offs_t dst, offs_t src, size_t size)
+void dbPagePool::copy(offs_t dst, offs_t src, offs_t size)
 {
     size_t dstOffs = (size_t)dst & (dbPageSize-1);
     size_t srcOffs = (size_t)src & (dbPageSize-1);
-    dst -= dstOffs;
-    src -= srcOffs;
+    dst -= (offs_t)dstOffs;
+    src -= (offs_t)srcOffs;
     byte* dstPage = find(dst, dbPageHeader::psDirty);
     byte* srcPage = find(src, 0);
     size = (size + 3) >> 2;
@@ -200,15 +200,39 @@ bool dbPagePool::open(dbFile* file, offs_t fileSize)
 
     if (poolSize == 0) { // autodetect pool size
         size_t totalMem = dbOSFile::ramSize();
-        for (poolSize = minPoolSize;
-             poolSize*2*dbPageSize < totalMem;
-             poolSize *= 2);
+        if (totalMem << 1 >> 1 != totalMem) { 
+            poolSize = (size_t)1 << (sizeof(totalMem)*8 - 2 - dbPageBits);
+        } else { 
+            for (poolSize = minPoolSize;
+                 poolSize*2*dbPageSize < totalMem;
+                 poolSize *= 2);
+        }
 #if 0
-        if (long(totalMem - poolSize*dbPageSize) > long(maxUnusedMemory)) {
+        if (size_t(totalMem - poolSize*dbPageSize) > size_t(maxUnusedMemory)) {
             poolSize = (totalMem - maxUnusedMemory) / dbPageSize;
         }
 #endif
     }
+    hashTable = NULL;
+    pages = NULL;
+
+    do { 
+#if defined(__WATCOMC__)
+        // reserve one more pages to allow access after end of page
+        bufferSize = (poolSize+1)*dbPageSize;
+#else
+        bufferSize = poolSize*dbPageSize;
+#endif
+        buffer = (byte*)dbOSFile::allocateBuffer(bufferSize, LOCK_PAGE_POOL_IN_MEMORY);
+        if (buffer != NULL) { 
+            break;
+        }
+        poolSize >>= 1;
+    } while (poolSize >= minPoolSize);
+
+    if (buffer == NULL) { 
+        return false;
+    }
 
     size_t hashSize;
     for (hashSize = minHashSize; hashSize < poolSize; hashSize *= 2);
@@ -218,7 +242,7 @@ bool dbPagePool::open(dbFile* file, offs_t fileSize)
 
     pages = new dbPageHeader[poolSize+1];
     pages->next = pages->prev = 0;
-    for (i = poolSize+1; --i != 0;) {
+    for (i = (int)poolSize+1; --i != 0;) {
         pages[i].state = 0;
         pages[i].next = i + 1;
     }
@@ -239,14 +263,7 @@ bool dbPagePool::open(dbFile* file, offs_t fileSize)
     nDirtyPages = 0;
     dirtyPages = new dbPageHeader*[poolSize];
 
-#if defined(__WATCOMC__)
-    // reserve one more pages to allow access after end of page
-    bufferSize = (poolSize+1)*dbPageSize;
-#else
-    bufferSize = poolSize*dbPageSize;
-#endif
-    buffer = (byte*)dbOSFile::allocateBuffer(bufferSize, LOCK_PAGE_POOL_IN_MEMORY);
-    return buffer != NULL;
+    return true;
 }
 
 
@@ -265,7 +282,7 @@ void dbPagePool::close()
 
 void dbPagePool::unfix(void* ptr)
 {
-    int i = (size_t((byte*)ptr - buffer) >> dbPageBits) + 1;
+    int i = (int)((size_t((byte*)ptr - buffer) >> dbPageBits) + 1);
     mutex.lock();
     dbPageHeader* ph = &pages[i];
     assert(ph->accessCount > 0);
@@ -284,7 +301,7 @@ void dbPagePool::unfix(void* ptr)
 
 void dbPagePool::unfixLIFO(void* ptr)
 {
-    int i = (size_t((byte*)ptr - buffer) >> dbPageBits) + 1;
+    int i = (int)((size_t((byte*)ptr - buffer) >> dbPageBits) + 1);
     mutex.lock();
     dbPageHeader* ph = &pages[i];
     assert(ph->accessCount > 0);
@@ -298,7 +315,7 @@ void dbPagePool::unfixLIFO(void* ptr)
 
 void dbPagePool::fix(void* ptr)
 {
-    int i = (size_t((byte*)ptr - buffer) >> dbPageBits) + 1;
+    size_t i = (size_t((byte*)ptr - buffer) >> dbPageBits) + 1;
     mutex.lock();
     dbPageHeader* ph = &pages[i];
     assert(ph->accessCount != 0);
@@ -309,14 +326,14 @@ void dbPagePool::fix(void* ptr)
 
 void dbPagePool::modify(void* ptr)
 {
-    int i = (size_t((byte*)ptr - buffer) >> dbPageBits) + 1;
+    size_t i = (size_t((byte*)ptr - buffer) >> dbPageBits) + 1;
     mutex.lock();
     dbPageHeader* ph = &pages[i];
     assert(ph->accessCount != 0);
     if (!(ph->state & dbPageHeader::psDirty)) {
         ph->state |= dbPageHeader::psDirty;
         dirtyPages[nDirtyPages] = ph;
-        ph->writeQueueIndex = nDirtyPages++;
+        ph->writeQueueIndex = (int4)(nDirtyPages++);
     }
     mutex.unlock();
 }
@@ -328,6 +345,23 @@ static int __cdecl compareOffs(void const* a, void const* b)
      return pa->offs < pb->offs ? -1 : pa->offs == pb->offs ? 0 : 1;
 }
 
+void dbPagePool::clear(offs_t fileSize)
+{
+    mutex.lock();
+    assert(nDirtyPages == 0);
+    pages->next = pages->prev = 0;
+    this->fileSize = fileSize;
+    for (int i = (int)poolSize+1; --i != 0;) {
+        pages[i].state = 0;
+        pages[i].next = i + 1;
+    }
+    pages[poolSize].next = 0;
+    freePages = 1;
+    memset(hashTable, 0, sizeof(int)*(hashBits+1));
+    nPages = 0;
+    mutex.unlock();
+}
+
 void dbPagePool::flush()
 {
     int rc;
@@ -336,7 +370,7 @@ void dbPagePool::flush()
     if (nDirtyPages != 0) {
         flushing = true;
         qsort(dirtyPages, nDirtyPages, sizeof(dbPageHeader*), compareOffs);
-        for (int i = 0, n = nDirtyPages; i < n; i++) {
+        for (int i = 0, n = (int)nDirtyPages; i < n; i++) {
             dbPageHeader* ph = dirtyPages[i];
             if (ph->accessCount++ == 0) {
                 pages[ph->next].prev = ph->prev;
@@ -361,7 +395,7 @@ void dbPagePool::flush()
             if (--ph->accessCount == 0) {
                 ph->next = pages->next;
                 ph->prev = 0;
-                pages->next = pages[ph->next].prev = (ph - pages);
+                pages->next = pages[ph->next].prev = (int)(ph - pages);
             }
         }
         flushing = false;
@@ -376,6 +410,38 @@ void dbPagePool::flush()
 }
 
 
+void dbGetTie::set(dbPagePool& pool, offs_t pos, size_t size)
+{
+    reset();
+    int offs = (int)pos & (dbPageSize-1);
+    byte* p = pool.get(pos - offs);
+    if (offs + size > dbPageSize) {
+        byte* dst = dbMalloc(size);
+        obj = dst;
+        memcpy(dst, p + offs, dbPageSize - offs);
+        pool.unfix(p);
+        size -= dbPageSize - offs;
+        pos += dbPageSize - offs;
+        dst += dbPageSize - offs;
+        while (size > dbPageSize) {
+            p = pool.get(pos);
+            memcpy(dst, p, dbPageSize);
+            dst += dbPageSize;
+            size -= dbPageSize;
+            pos += dbPageSize;
+            pool.unfix(p);
+        }
+        p = pool.get(pos);
+        memcpy(dst, p, size);
+        pool.unfix(p);
+        page = NULL;
+    } else {
+        this->pool = &pool;
+        page = p;
+        obj = p + offs;
+    }
+}
+
 void dbGetTie::set(dbPagePool& pool, offs_t pos)
 {
     reset();
@@ -409,6 +475,37 @@ void dbGetTie::set(dbPagePool& pool, offs_t pos)
     }
 }
 
+void dbGetTie::fetch(dbPagePool& pool, offs_t pos)
+{
+    reset();
+    int offs = (int)pos & (dbPageSize-1);
+    byte* p = pool.get(pos - offs);
+    size_t size = ((dbRecord*)(p + offs))->size;
+    byte* dst = dbMalloc(size);
+    obj = dst;
+    page = NULL;
+    if (offs + size > dbPageSize) {
+        memcpy(dst, p + offs, dbPageSize - offs);
+        pool.unfix(p);
+        size -= dbPageSize - offs;
+        pos += dbPageSize - offs;
+        dst += dbPageSize - offs;
+        while (size > dbPageSize) {
+            p = pool.get(pos);
+            memcpy(dst, p, dbPageSize);
+            dst += dbPageSize;
+            size -= dbPageSize;
+            pos += dbPageSize;
+            pool.unfix(p);
+        }
+        p = pool.get(pos);
+        memcpy(dst, p, size);
+        pool.unfix(p);
+    } else {
+        memcpy(dst, p + offs, size);
+    }
+}
+
 
 
 void dbGetTie::reset()
@@ -499,7 +596,9 @@ void dbPutTie::reset()
     }
     obj = NULL;
     if (oid != 0) { 
-        pool->db->updateCursors(oid);
+        if (updateCursors) { 
+            pool->db->updateCursors(oid);
+        }
         oid = 0;
     }
 }
