@@ -30,78 +30,80 @@ char_t* dbQueryElement::dump(char_t* buf)
 {
     switch (type) { 
       case qExpression:
-        buf += SPRINTF(buf, _T(" %s "), (char_t*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T(" %s "), (char_t*)ptr);
         break;
       case qVarBool:
-        buf += SPRINTF(buf, _T("{bool}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{bool}"));
         break;
       case qVarInt1:
-        buf += SPRINTF(buf, _T("{int1}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{int1}"));
         break;
       case qVarInt2:
-        buf += SPRINTF(buf, _T("{int2}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{int2}"));
         break;
       case qVarInt4:
-        buf += SPRINTF(buf, _T("{int4}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{int4}"));
         break;
       case qVarInt8:
-        buf += SPRINTF(buf, _T("{db_int8}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{db_int8}"));
         break;
       case qVarReal4:
-        buf += SPRINTF(buf, _T("{real4}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{real4}"));
         break;
       case qVarReal8:
-        buf += SPRINTF(buf, _T("{real8}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{real8}"));
         break;
       case qVarString:
-        buf += SPRINTF(buf, _T("{char*}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{char*}"));
         break;
       case qVarStringPtr:
-        buf += SPRINTF(buf, _T("{char**}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{char**}"));
         break;
       case qVarReference:
         if (ref != NULL) { 
-            buf += SPRINTF(buf, _T("{dbReference<%s>}"), ref->getName());
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbReference<%s>}"), ref->getName());
         } else { 
-            buf += SPRINTF(buf, _T("{dbAnyReference}"));
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbAnyReference}"));
         }
         break;
       case qVarArrayOfRef:
         if (ref != NULL) { 
-            buf += SPRINTF(buf, _T("{dbArray< dbReference<%s> >}"), ref->getName());
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbArray< dbReference<%s> >}"), ref->getName());
         } else { 
-            buf += SPRINTF(buf, _T("{dbArray<dbAnyReference>}"));
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbArray<dbAnyReference>}"));
         }
         break;
       case qVarArrayOfRefPtr:
         if (ref != NULL) { 
-            buf += SPRINTF(buf, _T("{dbArray< dbReference<%s> >*}"), ref->getName());
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbArray< dbReference<%s> >*}"), ref->getName());
         } else { 
-            buf += SPRINTF(buf, _T("{dbArray<dbAnyReference>*}"));
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbArray<dbAnyReference>*}"));
         }
         break;
       case qVarRawData:
-        buf += SPRINTF(buf, _T("{raw binary}"));
+      case qVarRawDataPtr:
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{raw binary}"));
         break;
       case qVarRectangle:
-        buf += SPRINTF(buf, _T("{rectangle}"));
+      case qVarRectanglePtr:
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{rectangle}"));
         break;
       case qVarUnknown:
-        buf += SPRINTF(buf, _T("???"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("???"));
         break;
 #ifdef USE_MFC_STRING
       case qVarMfcString:
-        buf += SPRINTF(buf, _T("{CString}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{CString}"));
         break;
 #endif  
 #ifdef USE_STD_STRING
       case qVarStdString:
-        buf += SPRINTF(buf, _T("{string}"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{string}"));
         break;
 #endif  
       default:
         break;
-   }
+    }
     return buf;
 }
         
@@ -109,82 +111,95 @@ char_t* dbQueryElement::dumpValues(char_t* buf)
 {
     switch (type) { 
       case qExpression:
-        buf += SPRINTF(buf, _T(" %s "), (char*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T(" %s "), (char*)ptr);
         break;
       case qVarBool:
-        buf += SPRINTF(buf, _T("%s"), *(bool*)ptr ? _T("true") : _T("false"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("%s"), *(bool*)ptr ? _T("true") : _T("false"));
         break;
       case qVarInt1:
-        buf += SPRINTF(buf, _T("%d"), *(int1*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("%d"), *(int1*)ptr);
         break;
       case qVarInt2:
-        buf += SPRINTF(buf, _T("%d"), *(int2*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("%d"), *(int2*)ptr);
         break;
       case qVarInt4:
-        buf += SPRINTF(buf, _T("%d"), *(int4*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("%d"), *(int4*)ptr);
         break;
       case qVarInt8:
-        buf += SPRINTF(buf, T_INT8_FORMAT, *(db_int8*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), T_INT8_FORMAT, *(db_int8*)ptr);
         break;
       case qVarReal4:
-        buf += SPRINTF(buf, _T("%f"), *(real4*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("%f"), *(real4*)ptr);
         break;
       case qVarReal8:
-        buf += SPRINTF(buf, _T("%f"), *(real8*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("%f"), *(real8*)ptr);
         break;
       case qVarString:
-        buf += SPRINTF(buf, _T("'%s'"), (char*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("'%s'"), (char*)ptr);
         break;
       case qVarStringPtr:
-        buf += SPRINTF(buf, _T("'%s'"), *(char**)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("'%s'"), *(char**)ptr);
         break;
       case qVarReference:
         if (ref != NULL) { 
-            buf += SPRINTF(buf, _T("@%s:%x"), ref->getName(), *(oid_t*)ptr);
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("@%s:%lx"), ref->getName(), (unsigned long)*(oid_t*)ptr);
         } else { 
-            buf += SPRINTF(buf, _T("@%x"), *(oid_t*)ptr);
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("@%lx"), (unsigned long)*(oid_t*)ptr);
         }
         break;
       case qVarArrayOfRef:
         if (ref != NULL) { 
-            buf += SPRINTF(buf, _T("{dbArray< dbReference<%s> >}"), ref->getName());
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbArray< dbReference<%s> >}"), ref->getName());
         } else { 
-            buf += SPRINTF(buf, _T("{dbArray<dbAnyReference>}"));
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbArray<dbAnyReference>}"));
         }
         break;
       case qVarArrayOfRefPtr:
         if (ref != NULL) { 
-            buf += SPRINTF(buf, _T("{dbArray< dbReference<%s> >*}"), ref->getName());
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbArray< dbReference<%s> >*}"), ref->getName());
         } else { 
-            buf += SPRINTF(buf, _T("{dbArray<dbAnyReference>*}"));
+            buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{dbArray<dbAnyReference>*}"));
         }
         break;
       case qVarRawData:
-        buf += SPRINTF(buf, _T("{raw binary}"));
+      case qVarRawDataPtr:
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("{raw binary}"));
         break;
       case qVarRectangle:
         {
             int i, sep = '(';
             rectangle& r = *(rectangle*)ptr;
             for (i = 0; i < rectangle::dim*2; i++) { 
-                buf += SPRINTF(buf, _T("%c%f"), sep, (double)r.boundary[i]);
+                buf += SPRINTF(SPRINTF_BUFFER(buf), _T("%c%f"), sep, (double)r.boundary[i]);
                 sep = ',';
             }
-            *buf = ')';
+            *buf++ = ')';
+            *buf = '\0';
+        }
+        break;
+      case qVarRectanglePtr:
+        {
+            int i, sep = '(';
+            rectangle& r = **(rectangle**)ptr;
+            for (i = 0; i < rectangle::dim*2; i++) { 
+                buf += SPRINTF(SPRINTF_BUFFER(buf), _T("%c%f"), sep, (double)r.boundary[i]);
+                sep = ',';
+            }
+            *buf++ = ')';
             *buf = '\0';
         }
         break;
       case qVarUnknown:
-        buf += SPRINTF(buf, _T("???"));
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("???"));
         break;
 #ifdef USE_MFC_STRING
       case qVarMfcString:
-        buf += SPRINTF(buf, _T("'%s'"), (LPCSTR)*(MFC_STRING*)ptr);
+        buf += SPRINTF(SPRINTF_BUFFER(buf), (sizeof(MFC_STRING::XCHAR)==sizeof(char)) ? _T("'%hs'") : _T("'%ls'"), (MFC_STRING::PCXSTR)*(MFC_STRING*)ptr);
         break;
 #endif  
 #ifdef USE_STD_STRING
       case qVarStdString:
-        buf += SPRINTF(buf, _T("'%s'"), ((STD_STRING*)ptr)->c_str());
+        buf += SPRINTF(SPRINTF_BUFFER(buf), _T("'%s'"), ((STD_STRING*)ptr)->c_str());
         break;
 #endif  
       default:
@@ -265,9 +280,6 @@ void dbCompiledQuery::destroy()
             delete tree;
             for (dbOrderByNode *op = order, *nop; op != NULL; op = nop) {
                 nop = op->next;
-                if (op->expr != NULL) { 
-                    delete op->expr;
-                }
                 delete op;
             }
             for (dbFollowByNode *fp = follow, *nfp; fp != NULL; fp = nfp) {
@@ -350,16 +362,24 @@ int dbUserFunction::getNumberOfParameters()
 dbUserFunction* dbUserFunction::list;
 
 
-void dbUserFunction::bind(char_t* fname, void* f, funcType ftype)
+void dbUserFunction::bind(char_t* name, void* f, funcType ftype)
 {
-    name = fname;
-    dbSymbolTable::add(name, tkn_ident, GB_CLONE_ANY_IDENTIFIER);
+    fname = name;
+    dbSymbolTable::add(fname, tkn_ident, GB_CLONE_ANY_IDENTIFIER);
     next = list;
     list = this;
     fptr = f;
     type = ftype;
 }
 
+dbUserFunction::~dbUserFunction()
+{
+    dbUserFunction *fp, **fpp;
+    for (fpp = &list; (fp = *fpp) != this; fpp = &fp->next);
+    *fpp = next;
+}
+        
+
 dbUserFunctionArgument::dbUserFunctionArgument(dbExprNode*             expr, 
                                                dbInheritedAttribute&   iattr, 
                                                dbSynthesizedAttribute& sattr, 
