@@ -13,12 +13,15 @@
 
 BEGIN_GIGABASE_NAMESPACE
 
+class dbSession;
+
 /**
  * Element of the query
  */
 class GIGABASE_DLL_ENTRY dbQueryElement {
     friend class dbQuery;
     friend class dbCompiler;
+    friend class dbSession;
     friend class dbQueryExpression;
     friend class dbQueryElementAllocator;
     friend class dbCLI;
@@ -36,9 +39,15 @@ class GIGABASE_DLL_ENTRY dbQueryElement {
         qVarStringPtr,
         qVarReference,
         qVarRectangle, 
-        qVarArrayOfRef,
-        qVarArrayOfRefPtr, 
+        qVarRectanglePtr, 
+        qVarArrayOfRef, 
+        qVarArrayOfInt4,
+        qVarArrayOfInt8,
+        qVarArrayOfRefPtr,
+        qVarArrayOfInt4Ptr,
+        qVarArrayOfInt8Ptr,
         qVarRawData,
+        qVarRawDataPtr,
         qVarStdString,
         qVarMfcString,
         qVarUnknown
@@ -180,12 +189,12 @@ class GIGABASE_DLL_ENTRY dbQueryExpression {
     dbQueryExpression& operator,(db_nat8 const& ptr) {
         return add(dbQueryElement::qVarInt8, &ptr);
     }
-#if SIZEOF_LONG != 8
+#if SIZEOF_LONG != 8 
     dbQueryExpression& operator,(long const& ptr) {
-        return add(dbQueryElement::qVarInt4, &ptr);
+        return add(sizeof(long) == 4 ? dbQueryElement::qVarInt4 : dbQueryElement::qVarInt8, &ptr);
     }
     dbQueryExpression& operator,(unsigned long const& ptr) {
-        return add(dbQueryElement::qVarInt4, &ptr);
+        return add(sizeof(long) == 4 ? dbQueryElement::qVarInt4 : dbQueryElement::qVarInt8, &ptr);
     }
 #endif
     dbQueryExpression& operator,(real4 const& ptr) {
@@ -197,6 +206,9 @@ class GIGABASE_DLL_ENTRY dbQueryExpression {
     dbQueryExpression& operator,(void const* ptr) { 
         return add(dbQueryElement::qVarRawData, ptr);
     }
+    dbQueryExpression& operator,(void** ptr) { 
+        return add(dbQueryElement::qVarRawDataPtr, ptr);
+    }
 #ifndef bool
     dbQueryExpression& operator,(bool const& ptr) {
         return add(dbQueryElement::qVarBool, &ptr);
@@ -215,6 +227,9 @@ class GIGABASE_DLL_ENTRY dbQueryExpression {
     dbQueryExpression& operator,(rectangle const& rect) {
         return add(dbQueryElement::qVarRectangle, &rect);
     }
+    dbQueryExpression& operator,(rectangle* rect) {
+        return add(dbQueryElement::qVarRectanglePtr, &rect);
+    }
 #ifdef USE_STD_STRING
     dbQueryExpression& operator,(STD_STRING const& str) {
         return add(dbQueryElement::qVarStdString, &str);
@@ -267,8 +282,26 @@ class GIGABASE_DLL_ENTRY dbQueryExpression {
         return add(dbQueryElement::qVarArrayOfRefPtr, &value, &T::dbDescriptor);
     }
 #endif
+#if !defined(_MSC_VER) || _MSC_VER+0 >= 1300
+    inline dbQueryExpression& operator,(dbArray<db_int4> const& value) { 
+        return add(dbQueryElement::qVarArrayOfInt4, &value);
+    }
+
+    inline dbQueryExpression& operator,(dbArray<db_int4>const* const& value) { 
+        return add(dbQueryElement::qVarArrayOfInt4Ptr, &value);
+    }
+
+    inline dbQueryExpression& operator,(dbArray<db_int8> const& value) { 
+        return add(dbQueryElement::qVarArrayOfInt8, &value);
+    }
+
+    inline dbQueryExpression& operator,(dbArray<db_int8>const* const& value) { 
+        return add(dbQueryElement::qVarArrayOfInt8Ptr, &value);
+    }
+#endif
 };
 
+class dbExprNode;
 class dbOrderByNode;
 class dbFollowByNode;
 
@@ -283,10 +316,11 @@ class GIGABASE_DLL_ENTRY dbCompiledQuery {
     dbTableDescriptor* table;
     int                schemeVersion;
 
-    size_t             stmtLimitStart;
-    size_t             stmtLimitLen;
-    int4*              stmtLimitStartPtr;
-    int4*              stmtLimitLenPtr;
+    cardinality_t  
+    stmtLimitStart;
+    cardinality_t      stmtLimitLen;
+    cardinality_t*     stmtLimitStartPtr;
+    cardinality_t*     stmtLimitLenPtr;
     bool               limitSpecified;
 
     enum IteratorInit {
@@ -329,6 +363,7 @@ class GIGABASE_DLL_ENTRY dbCompiledQuery {
 class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     friend class dbCompiler;
     friend class dbDatabase;
+    friend class dbSession;
     friend class dbSubSql;
     friend class dbCLI;
   private:
@@ -340,12 +375,14 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     //
     // Prohibite query copying
     //
-    dbQuery(dbQuery const&) {}
+    dbQuery(dbQuery const&) : dbCompiledQuery() {}
     dbQuery& operator =(dbQuery const&) { return *this; }
 
   public:
     int                pos; // position of condition in statement
 
+    bool compileError() { return !isEmpty() && !compiled(); }
+
     char_t* dump(char_t* buf) { 
         char_t* p = buf;
         for (dbQueryElement* elem = elements; elem != NULL; elem = elem->next) { 
@@ -370,6 +407,12 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
         return *this;
     }
 
+    bool isEmpty() { 
+        return elements == NULL 
+            || (elements->next == NULL && elements->type == dbQueryElement::qExpression && *(char_t*)elements->ptr == 0);
+    }
+
+
     dbQuery& reset();
 
     //
@@ -389,6 +432,20 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
 
     dbQuery& add(dbQueryExpression const& expr);
 
+    dbQuery& And(dbQueryExpression const& expr) { 
+        if (elements != NULL) { 
+            append(dbQueryElement::qExpression, _T("and"));
+        }
+        return add(expr);
+    }
+
+    dbQuery& Or(dbQueryExpression const& expr) { 
+        if (elements != NULL) { 
+            append(dbQueryElement::qExpression, _T("or"));
+        }
+        return add(expr);
+    }
+
     dbQuery& And(char_t const* str) {
         if (elements != NULL) {
             append(dbQueryElement::qExpression, _T("and"));
@@ -416,6 +473,9 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     dbQuery& add(rectangle const& rect) {
         return append(dbQueryElement::qVarRectangle, &rect);
     }
+    dbQuery& add(rectangle* rect) {
+        return append(dbQueryElement::qVarRectanglePtr, &rect);
+    }
 #ifdef USE_STD_STRING
     dbQuery& add(STD_STRING const& str) { 
         return append(dbQueryElement::qVarStdString, &str);
@@ -452,12 +512,12 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     dbQuery& add (db_nat8 const& value) {
         return append(dbQueryElement::qVarInt8, &value);
     }
-#if SIZEOF_LONG != 8
+#if SIZEOF_LONG != 8 
     dbQuery& add (long const& value) {
-        return append(dbQueryElement::qVarInt4, &value);
+        return append(sizeof(long) == 4 ? dbQueryElement::qVarInt4 : dbQueryElement::qVarInt8, &value);
     }
     dbQuery& add (unsigned long const& value) {
-        return append(dbQueryElement::qVarInt4, &value);
+        return append(sizeof(long) == 4 ? dbQueryElement::qVarInt4 : dbQueryElement::qVarInt8, &value);
     }
 #endif
     dbQuery& add (real4 const& value) {
@@ -469,6 +529,9 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     dbQuery& add(void const* value) { 
         return append(dbQueryElement::qVarRawData, value);
     }
+    dbQuery& add(void** value) { 
+        return append(dbQueryElement::qVarRawDataPtr, value);
+    }
 #ifndef bool
     dbQuery& add(bool const& value) {
         return append(dbQueryElement::qVarBool, &value);
@@ -501,6 +564,40 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
         return reset().append(dbQueryElement::qExpression, str);
     }
 
+#if !defined(_MSC_VER) || _MSC_VER+0 >= 1300
+    inline dbQuery& operator,(dbArray<db_int4> const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt4, &value);
+    }
+
+    inline dbQuery& operator,(dbArray<db_int4>const* const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt4Ptr, &value);
+    }
+
+    inline dbQuery& operator,(dbArray<db_int8> const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt8, &value);
+    }
+
+    inline dbQuery& operator,(dbArray<db_int8>const* const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt8Ptr, &value);
+    }
+
+    inline dbQuery& add(dbArray<db_int4> const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt4, &value);
+    }
+
+    inline dbQuery& add(dbArray<db_int4>const* const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt4Ptr, &value);
+    }
+
+    inline dbQuery& add(dbArray<db_int8> const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt8, &value);
+    }
+
+    inline dbQuery& add(dbArray<db_int8>const* const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt8Ptr, &value);
+    }
+#endif
+
 #ifndef NO_MEMBER_TEMPLATES
     template<class T>
     dbQuery& operator,(dbReference<T> const& value) { 
@@ -570,8 +667,7 @@ inline dbQueryExpression& operator,(dbQueryExpression& expr, dbReference<T> cons
 }
 template<class T>
 inline dbQueryExpression& operator,(dbQueryExpression& expr, dbArray< dbReference<T> > const& value) { 
-    return expr.add(dbQueryElement::qVarArrayOfRef, &value, 
-                    &T::dbDescriptor);
+    return expr.add(dbQueryElement::qVarArrayOfRef, &value, &T::dbDescriptor);
 }
 
 template<class T>
@@ -581,24 +677,20 @@ inline dbQueryExpression& operator,(dbQueryExpression& expr, dbArray< dbReferenc
 
 template<class T>
 inline dbQuery& operator,(dbQuery& query, dbReference<T> const& value) { 
-    return query.append(dbQueryElement::qVarReference, &value, 
-                        &T::dbDescriptor);
+    return query.append(dbQueryElement::qVarReference, &value, &T::dbDescriptor);
 }
 
 template<class T>
-inline dbQuery& operator,(dbQuery& query, 
-                             dbArray< dbReference<T> > const& value) 
+inline dbQuery& operator,(dbQuery& query, dbArray< dbReference<T> > const& value) 
 { 
-    return query.append(dbQueryElement::qVarArrayOfRef, &value, 
-                        &T::dbDescriptor);
+    return query.append(dbQueryElement::qVarArrayOfRef, &value, &T::dbDescriptor);
 }
 
 template<class T>
 inline dbQuery& operator,(dbQuery& query, 
                           dbArray< dbReference<T> >const* const& value) 
 { 
-    return query.append(dbQueryElement::qVarArrayOfRefPtr, &value, 
-                        &T::dbDescriptor);
+    return query.append(dbQueryElement::qVarArrayOfRefPtr, &value, &T::dbDescriptor);
 }
 
 template<class T>
@@ -608,8 +700,7 @@ inline dbQuery& add(dbQuery& query, dbReference<T> const& value) {
 
 template<class T>
 inline dbQuery& add(dbQuery& query, dbArray< dbReference<T> > const& value) { 
-    return query.append(dbQueryElement::qVarArrayOfRef, &value, 
-                        &T::dbDescriptor);
+    return query.append(dbQueryElement::qVarArrayOfRef, &value, &T::dbDescriptor);
 }
 
 template<class T>
@@ -668,7 +759,7 @@ class GIGABASE_DLL_ENTRY dbUserFunction {
     friend class dbCompiler;
 
     void*   fptr;
-    char_t* name;
+    char_t* fname;
 
     dbUserFunction* next;
     static dbUserFunction* list;
@@ -707,7 +798,7 @@ class GIGABASE_DLL_ENTRY dbUserFunction {
 
     static dbUserFunction* find(char_t const* name) {
         for (dbUserFunction* func = list; func != NULL; func = func->next) {
-            if (name == func->name) {
+            if (name == func->fname) {
                 return func;
             }
         }
@@ -794,6 +885,8 @@ class GIGABASE_DLL_ENTRY dbUserFunction {
     dbUserFunction(real8 (__cdecl *f)(dbUserFunctionArgument&, dbUserFunctionArgument&, dbUserFunctionArgument&), char_t* name) { 
         bind(name, (void*)f, fArgArgArg2Real); 
     }
+
+    ~dbUserFunction();
 };
 
 END_GIGABASE_NAMESPACE
