@@ -11,13 +11,13 @@
 #include "gigabase.h"
 #include <stdio.h>
 
-#ifdef ENCRYPTED
-#include "crypt_file.h"
+#ifdef SUPPORT_DATA_ENCRYPTION
+#include "crypt/crypt_file.h"
 #endif
 
 USE_GIGABASE_NAMESPACE
 
-const int nRecords = 100000;
+const int nRecords = 1000000;
 const int nSequentialSearches = 10;
 
 class Record {
@@ -41,12 +41,12 @@ int __cdecl main(int argc, char* argv[])
         nThreads = atoi(argv[1]);
     }
     dbDatabase db(dbDatabase::dbAllAccess, 5*1024); // 40Mb page pool
-#ifdef ENCRYPTED 
+#ifdef SUPPORT_DATA_ENCRYPTION 
     dbCryptFile cf("KEY");
     int rc = cf.open(_T("testperf.dbs"), dbFile::no_buffering);
     if (rc != dbFile::ok) { 
         char_t buf[256];
-        cf.errorText(rc, buf, sizeof buf);
+        cf.errorText(rc, buf, itemsof(buf));
         FPRINTF(stderr, _T("Failed to open encrupted file: %s\n"), buf);
         return EXIT_FAILURE;
     }
@@ -59,11 +59,13 @@ int __cdecl main(int argc, char* argv[])
         dbCursor<Record> cursor;
         q = _T("key="),buf;
         nat8 key = 1999;
+        nat8 sum = 0;
         time_t start = time(NULL);
         for (i = 0; i < nRecords; i++) {
             Record rec;
             key = (3141592621u*key + 2718281829u) % 1000000007u;
-            SPRINTF(buf, T_INT8_FORMAT _T("."), key);
+            sum += key;
+            SPRINTF(SPRINTF_BUFFER(buf), T_INT8_FORMAT _T("."), key);
             rec.key = buf;
             rec.value = buf;
             insert(rec);
@@ -78,9 +80,10 @@ int __cdecl main(int argc, char* argv[])
         key = 1999;
         for (i = 0; i < nRecords; i++) {
             key = (3141592621u*key + 2718281829u) % 1000000007u;
-            SPRINTF(buf, T_INT8_FORMAT _T("."), key);
+            SPRINTF(SPRINTF_BUFFER(buf), T_INT8_FORMAT _T("."), key);
             n = cursor.select(q);
             assert(n == 1);
+            assert(STRCMP(cursor->key, buf) == 0);
         }
         printf("Elapsed time for %d index searches: %d seconds\n",
                nRecords, int(time(NULL) - start));
@@ -89,6 +92,10 @@ int __cdecl main(int argc, char* argv[])
         for (i = 0; i < nSequentialSearches; i++) {
             n = cursor.select(q);
             assert(n == nRecords);
+            do {
+                n -= 1;
+            } while (cursor.next());
+            assert(n == 0);
         }
         printf("Elapsed time for %d sequential search through %d records: "
                "%d seconds\n", nSequentialSearches, nRecords,
