@@ -9,6 +9,9 @@
 //-------------------------------------------------------------------*--------*
 
 #ifndef _WIN32
+#define _INCLUDE_HPUX_SOURCE
+#define __EXTENSIONS__
+#define _EXTENSIONS
 
 #include "unisock.h"
 #undef BYTE_ORDER
@@ -25,7 +28,9 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/utsname.h>
+#ifndef HPUX11
 #include <sys/select.h>
+#endif
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
@@ -49,9 +54,10 @@ extern "C" {
 
 BEGIN_GIGABASE_NAMESPACE
 
-#define MAX_HOST_NAME     256
+const int MAX_HOST_NAME = 256;
+const int GETHOSTBYNAME_BUF_SIZE = 1024;
 
-char* unix_socket::unix_socket_dir = "/tmp/";
+char* unix_socket::unix_socket_dir = (char*)"/tmp/";
 
 class unix_socket_library {
   public:
@@ -104,10 +110,23 @@ bool unix_socket::open(int listen_queue_size)
     } else {
         u.sock_inet.sin_family = AF_INET;
         if (*hostname && strcmp(hostname, "localhost") != 0) {
-            struct hostent* hp;  // entry in hosts table
-            if ((hp = gethostbyname(hostname)) == NULL
+            struct hostent* hp;
+#if defined(HAVE_GETHOSTBYNAME_R) && !defined(NO_PTHREADS)
+            struct hostent ent;  // entry in hosts table
+            char buf[GETHOSTBYNAME_BUF_SIZE];
+            int h_err;
+#if defined(__sun)
+            if ((hp = gethostbyname_r(hostname, &ent, buf, sizeof buf, &h_err)) == NULL
+#else
+            if (gethostbyname_r(hostname, &ent, buf, sizeof buf, &hp, &h_err) != 0
+                || hp == NULL
+#endif
                 || hp->h_addrtype != AF_INET)
-            {
+#else
+            if ((hp = gethostbyname(hostname)) == NULL || hp->h_addrtype != AF_INET) 
+#endif
+            {                   
+                TRACE_MSG(("Failed to get host by name: %s\n", errno));
                 errcode = bad_address;
                 return false;
             }
@@ -148,7 +167,7 @@ char* unix_socket::get_peer_name()
         return NULL;
     }
     struct sockaddr_in insock;
-#if defined(__linux__) || (defined(__FreeBSD__) && __FreeBSD__ > 3) || defined(_AIX43) || defined(__OpenBSD__)
+#if defined(__linux__) || (defined(__FreeBSD__) && __FreeBSD__ > 3) || defined(_AIX43) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(HPUX11) || defined (__DragonFly__) || defined(_SOCKLEN_T)
     socklen_t len = sizeof(insock);
 #elif defined(_AIX41)
     size_t len = sizeof(insock);
@@ -227,7 +246,6 @@ socket_t* unix_socket::accept()
         errcode = not_opened;
         return NULL;
     } else {
-        static struct linger l = {1, LINGER_TIME};
         if (domain == sock_global_domain) {
             int enabled = 1;
             if (setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (char*)&enabled,
@@ -238,11 +256,14 @@ socket_t* unix_socket::accept()
                 return NULL;
             }
         }
+#ifdef USE_SO_LINGER
+        static struct linger l = {1, LINGER_TIME};
         if (setsockopt(s, SOL_SOCKET, SO_LINGER, (char*)&l, sizeof l) != 0) {
             errcode = invalid_access_mode;
             ::close(s);
             return NULL;
         }
+#endif
         errcode = ok;
         return new unix_socket(s);
     }
@@ -326,10 +347,23 @@ bool unix_socket::connect(int max_attempts, time_t timeout)
         }
     } else {
         sockaddr_in sock_inet;
-        struct hostent* hp;  // entry in hosts table
-
-        if ((hp=gethostbyname(hostname)) == NULL || hp->h_addrtype != AF_INET)
+        struct hostent* hp;
+#if defined(HAVE_GETHOSTBYNAME_R) && !defined(NO_PTHREADS)
+        struct hostent ent;  // entry in hosts table
+        char buf[GETHOSTBYNAME_BUF_SIZE];
+        int h_err;
+#if defined(__sun)
+        if ((hp = gethostbyname_r(hostname, &ent, buf, sizeof buf, &h_err)) == NULL
+#else
+        if (gethostbyname_r(hostname, &ent, buf, sizeof buf, &hp, &h_err) != 0
+            || hp == NULL
+#endif
+            || hp->h_addrtype != AF_INET)
+#else
+        if ((hp = gethostbyname(hostname)) == NULL || hp->h_addrtype != AF_INET) 
+#endif
         {
+            TRACE_MSG(("Host name can not be resolved: %d\n", errno));
             errcode = bad_address;
             return false;
         }
@@ -363,6 +397,14 @@ bool unix_socket::connect(int max_attempts, time_t timeout)
                         ::close(fd);
                         return false;
                     }
+#ifdef USE_SO_LINGER
+                    static struct linger l = {1, LINGER_TIME};
+                    if (setsockopt(fd, SOL_SOCKET, SO_LINGER, (char*)&l, sizeof l) != 0) {
+                        errcode = invalid_access_mode;
+                        ::close(fd);
+                        return NULL;
+                    }
+#endif
                     errcode = ok;
                     state = ss_open;
                     return true;
@@ -494,7 +536,7 @@ unix_socket::~unix_socket()
     if (create_file) {
         char name[MAX_HOST_NAME];
         char* p = strrchr(address, ':');
-        sprintf(name, "%s%.*s.%s", unix_socket_dir, p - address, address, p+1);
+        sprintf(name, "%s%.*s.%s", unix_socket_dir, (int)(p - address), address, p+1);
         unlink(name);
     }
     delete[] address;
