@@ -15,6 +15,7 @@
 BEGIN_GIGABASE_NAMESPACE
 
 const size_t init_reply_buffer_size = 1024;
+const time_t ACCEPT_FAILURE_TIMEOUT = 1; // second
 
 #define hhex(ch) (((ch) >> 4) >= 10 ? ((ch) >> 4) + 'A' - 10 : ((ch) >> 4) + '0')
 #define lhex(ch)  (((ch) & 0xF) >= 10 ? ((ch) & 0xF) + 'A' - 10 : ((ch) & 0xF) + '0')
@@ -97,7 +98,7 @@ bool WWWconnection::terminatedBy(char const* str) const
 }
 
 WWWconnection& WWWconnection::append(const void *buf, int len) {
-    int pos = reply_buf_used;
+    int pos = (int)reply_buf_used;
     char *dst = extendBuffer(len);
     memcpy(dst + pos, buf, len);
     return *this;
@@ -106,7 +107,7 @@ WWWconnection& WWWconnection::append(const void *buf, int len) {
 
 WWWconnection& WWWconnection::append(char const* str)
 {
-    int pos = reply_buf_used;
+    int pos = (int)reply_buf_used;
     char* dst = extendBuffer(strlen(str));
     unsigned char ch;
     switch (encoding) {
@@ -189,7 +190,7 @@ WWWconnection& WWWconnection::append(char const* str)
                     sprintf(buf, "&#%d;", ch);
                     dst = extendBuffer(strlen(buf)-1); // one already done
                     strcpy(dst + pos, buf);
-                    pos+=strlen(buf);
+                    pos += (int)strlen(buf);
                 }
                 break;
               case '\0':          
@@ -259,6 +260,7 @@ WWWconnection& WWWconnection::append(wchar_t const* str)
                 dst[pos++] = 0x80 | ((ch >> 6) & 0x3F);
                 dst[pos++] = 0x80 | (ch & 0x3F);
             }
+#ifndef _WIN32
             else if (ch <= 0x1FFFFF)
             {
                 dst = extendBuffer(3);
@@ -275,7 +277,8 @@ WWWconnection& WWWconnection::append(wchar_t const* str)
                 dst[pos++] = 0x80 | ((ch >> 12) & 0x3F);
                 dst[pos++] = 0x80 | ((ch >> 6) & 0x3F);
                 dst[pos++] = 0x80 | (ch & 0x3F);
-            }          
+            } 
+#endif         
         }      
         encoding = HTML;
         break;
@@ -334,6 +337,7 @@ WWWconnection& WWWconnection::append(wchar_t const* str)
                     dst[pos++] = 0x80 | ((ch >> 6) & 0x3F);
                     dst[pos++] = 0x80 | (ch & 0x3F);
                 }
+#ifndef _WIN32
                 else if (ch <= 0x1FFFFF)
                 {
                     dst = extendBuffer(3);
@@ -351,6 +355,7 @@ WWWconnection& WWWconnection::append(wchar_t const* str)
                     dst[pos++] = 0x80 | ((ch >> 6) & 0x3F);
                     dst[pos++] = 0x80 | (ch & 0x3F);
                 }
+#endif
             }
         }
         break;
@@ -428,6 +433,7 @@ WWWconnection& WWWconnection::append(wchar_t const* str)
                     dst[pos++] = hhex(0x80 | (ch & 0x3F));
                     dst[pos++] = lhex(0x80 | (ch & 0x3F));              
                 }
+#ifndef _WIN32
                 else if (ch <= 0x1FFFFF)
                 {
                     dst[pos++] = hhex(0xF0 | (ch >> 18));
@@ -461,6 +467,7 @@ WWWconnection& WWWconnection::append(wchar_t const* str)
                     dst[pos++] = hhex(0x80 | (ch & 0x3F));
                     dst[pos++] = lhex(0x80 | (ch & 0x3F));   
                 }
+#endif
             } else {
                 dst[pos++] = (char)ch;
             }
@@ -609,7 +616,7 @@ bool WWWapi::open(char const* socket_address,
     canceled = false;
     if (!sock->is_ok()) {
         char_t buf[64];
-        sock->get_error_text(buf, sizeof buf);
+        sock->get_error_text(buf, itemsof(buf));
         FPRINTF(stderr, _T("WWWapi::open: create socket failed: %s\n"), buf);
         return false;
     }
@@ -629,7 +636,7 @@ bool WWWapi::connect(WWWconnection& con)
     if (con.sock == NULL) {
         if (!canceled) {
             char_t buf[64];
-            sock->get_error_text(buf, sizeof buf);
+            sock->get_error_text(buf, itemsof(buf));
             FPRINTF(stderr, _T("WWWapi::connect: accept failed: %s\n"), buf);
         }
         return false;
@@ -676,7 +683,8 @@ void URL2ASCII(char* src)
     char ch;
     while ((ch = *src++) != '\0') {
         if (ch == '%') {
-            *dst++ = ((src[0] - '0') << 8) | (src[1] - '0');
+            *dst++ = (HEX_DIGIT(src[0]) << 4) | HEX_DIGIT(src[1]);
+            src += 2;
         } else if (ch == '+') {
             *dst++ = ' ';
         } else if (ch == '.' && *src == '.') {
@@ -703,6 +711,7 @@ bool CGIapi::serve(WWWconnection& con)
     char* buf = new char[size];
     if (con.sock->read(buf, size, size) != size)
     {
+        delete[] buf;
         return true;
     }
     char* page = con.unpack(buf + buf[0], length - sizeof length - buf[0]);
@@ -714,13 +723,15 @@ bool CGIapi::serve(WWWconnection& con)
     {
         con.extendBuffer(4);
         result = dispatch(con, page);
-        *(int4*)con.reply_buf = con.reply_buf_used;
+        *(int4*)con.reply_buf = (int4)con.reply_buf_used;
         con.sock->write(con.reply_buf, con.reply_buf_used);
     }
     delete con.sock;
     con.sock = NULL; // close connection
     delete[] con.peer;
     con.peer = NULL;
+    con.reset();
+    delete[] buf; 
     return result;
 }
 
@@ -881,8 +892,8 @@ bool HTTPapi::serve(WWWconnection& con)
                     size_t file_size = ftell(f);
                     fseek(f, 0, SEEK_SET);
                     char reply[1024];
-                    char* contentType;
-                    char* contentSubtype;
+                    char const* contentType;
+                    char const* contentSubtype;
                     char* suf = strrchr(file, '.');
                     if (suf != NULL) { 
                         if (strcmp(suf+1, "htm") == 0 || strcmp(suf+1, "html") == 0) { 
@@ -911,9 +922,9 @@ bool HTTPapi::serve(WWWconnection& con)
                         contentSubtype = "binary";
                         contentType = "image";
                     }
-                    sprintf(reply, "HTTP/1.1 200 OK\r\nContent-Length: %u\r\n"
+                    sprintf(reply, "HTTP/1.1 200 OK\r\nContent-Length: %lu\r\n"
                             "Content-Type: %s/%s\r\nConnection: %s\r\n\r\n",
-                            file_size, contentType, contentSubtype, 
+                            (long)file_size, contentType, contentSubtype, 
                             keepConnectionAlive ? "Keep-Alive" : "close");
                     con.append(reply);
                     size_t pos = con.reply_buf_used;
@@ -945,7 +956,7 @@ bool HTTPapi::serve(WWWconnection& con)
             char* body = p;
           ScanNextPart:
             int n = length < buf + size - p
-                ? length : buf + size - p;
+                ? length : (int)(buf + size - p);
             while (--n >= 0 && *p && *p != '\r' && *p != '\n')
             {
                 p += 1;
@@ -1010,7 +1021,7 @@ bool HTTPapi::serve(WWWconnection& con)
             delete[] buf;
             return true;
         }
-        if (p - buf < (long)size) {
+        if ((size_t)(p - buf) < size) {
             size -= p - buf;
             memcpy(buf, p, size);
         } else {
@@ -1034,13 +1045,10 @@ bool HTTPapi::handleRequest(WWWconnection& con, char* begin, char* end,
                             char* host, bool& result)
 {
     char buf[64];
-    char ch = *end;
-  
-    *buf ='\0';
     char* page = con.unpack(begin, end - begin);
     if (page != NULL)  {
         con.append("HTTP/1.1 200 OK\r\nContent-Length:           \r\n");
-        int length_pos = con.reply_buf_used - 12;
+        int length_pos = (int)con.reply_buf_used - 12;
         con.append(keepConnectionAlive
                    ? "Connection: Keep-Alive\r\n"
                    : "Connection: close\r\n");
@@ -1063,19 +1071,17 @@ bool HTTPapi::handleRequest(WWWconnection& con, char* begin, char* end,
             return false;
         }
         body += *body == '\n' ? 1 : 2;
-        sprintf(buf, "%u", (unsigned)(con.reply_buf_used - (body - con.reply_buf)));
+        sprintf(buf, "%lu", (long)(con.reply_buf_used - (body - con.reply_buf)));
         memcpy(con.reply_buf + length_pos,
                buf, strlen(buf));    
         if (!con.sock->write(con.reply_buf, con.reply_buf_used)) {
             return false;
         }
-        *end = ch;
         return result && keepConnectionAlive;
     } else {
         con.append(ERROR_TEXT("406 Not acceptable"));
         con.sock->write(con.reply_buf, con.reply_buf_used);
         result = true;
-        *end = ch;
         return false;
     }
 }
@@ -1154,12 +1160,19 @@ void QueueManager::start()
         WWWapi* srv = server;
         mutex.unlock();
         if (!srv->connect(*con) || server == NULL) {
-            return;
+            dbThread::sleep(ACCEPT_FAILURE_TIMEOUT);
+            mutex.lock();
+            con->next = freeList;
+            freeList = con;
+            if (server == NULL) { 
+                break;
+            }
+        } else { 
+            mutex.lock();
+            con->next = waitList;
+            waitList = con;
+            go.signal();
         }
-        mutex.lock();
-        con->next = waitList;
-        waitList = con;
-        go.signal();
     }
     mutex.unlock();
 }
